# from typing import Optional
# import json
# import modal
# import pydantic  # for typing, used later
# from typing import Optional, Dict, List, Union, Any
# import uuid
# import os
# from fastapi import FastAPI, WebSocket
# import random
# import string
# import re
# from typing import Dict, Any
# import asyncio
# from pydantic import BaseModel, Field
# import time
# import httpx
# from fastapi import Request, HTTPException
# from openai import OpenAI
# from Functions.helper import log_time
# from Functions.process_single_question_wrapper import *
# from Question_TypesV2.number_of_question import *
# from Question_TypesV2.split_questions import split_questions
# import httpx
# from Functions.images import Input_FE
# from asyncio import to_thread
# from concurrent.futures import ProcessPoolExecutor
# from Functions.images import *
# from Functions.check_user_status import check_user_status
# from Functions.accept_suggestion import accept_suggestion
# from typing import List, Optional, Literal
# import asyncio
# import time
# import pydantic
# from modal import web_endpoint, Secret, app
# from fastapi import Request
# from Functions.router import router_prompt
# from Cursor.general_chat import general_chat
# from Cursor.add_question import add_question
# from Cursor.add_section import add_section
# from Cursor.delete_a_question import delete_question
# from Cursor.find_question_text_from_label import find_question_text_by_label
# from CreateSection.Functions.annotation import annotation
# from CreateSection.Functions.process_questions_in_parallel import process_questions_parallel
# from CreateSection.Functions.transform_survey_data import *
# from CreateSection.Functions.image import survey_generator
# from CreateSection.Functions.identify_questions import SurveyGen
# from CreateSection.Functions.identify_questions import *
# from Cursor.generate_new_label import generate_next_question_label
# from Cursor.delete_question_from_payload import delete_question_by_label
# from CreateSection.Functions.extract_survey_questions import extract_survey_questions
# from Edit_Selection.prompts.correction import correction_prompt
# from CreateSection.Functions.get_block_names import get_block_names
# from Functions.get_complete_questions import generate_complete_questions
# from Cursor.finalize_objective import finalize_survey_objective
# from Cursor.objective import determine_objective
# from Functions.create_conversation import extract_conversation_from_metadata
# from Cursor.recommend_sections import recommend_sections
# from Functions.extract_question_labels import extract_question_labels
# from Cursor.edit_from_chat import correction_prompt_via_chat
# from Functions.get_question_data_from_label import find_question_by_label
# import requests

# from Functions.recommend_provider_segments import recommend_provider_segments
# import websockets
# from starlette.websockets import WebSocketState
# from Cursor.budget_details import budget_details
# from Functions.router_publshed import router_prompt_published
# from Functions.extract_block_names import extract_block_names
# from Functions.delete_sections import delete_sections
# from Functions.optimize_survey_instructions import optimize_survey_instructions
# from Functions.get_complete_payload import generate_complete_questions_payload
# from Cursor.recommed_sections_to_optimize import recommend_sections_to_optimize
# from Cursor.add_section_for_optimize import add_section_for_optimize
# from Functions.estimate_time import estimate_time
# from Functions.request_bid import request_bid
# from Cursor.validate import validate_survey
# from Functions.get_raw_data import get_raw_data
# from Cursor.fix_post_validation import fix_post_validation
# from Functions.finalize_bid import finalize_bid
# from Functions.extract_first_section import extract_first_section
# from Cursor.quota import quota_creator

# app = modal.App("Create Survey")


# with open("CreateSection/Functions/screener_section.json", "r") as f:
#     readymade_screener_section = json.load(f)

# @app.cls(
#     image=categorizer,
#     secrets=[modal.Secret.from_name("openai-secret")]
#     # enable_memory_snapshot=True
# )
# class OpenAIModel:

#     def __init__(self):
#         self.client_open_ai = OpenAI()
 
        
        

# @app.function(image=testing_image, secrets=[modal.Secret.from_name("openai-secret")], timeout=2000, min_containers=1)
# @modal.asgi_app(label="create-survey")
# def create_survey():
#     fastapi_app = FastAPI()
#     client = OpenAI()
#     survey_gen = SurveyGen()
    

#     @fastapi_app.websocket("/ws")
#     async def new_text_editor_v2(websocket: WebSocket):
#         await websocket.accept()
#         objective_finalized = False
#         current_chances = 1
#         final_user_objective = None
#         refined_user_objective = None
#         change_in_sample_specs = False
#         openai_model = OpenAIModel()

#         sample_specs={
#                         "target_audience": "",
                        
#                     }
#         try:
#             # Infinite loop to keep the connection alive
#             while True:
#                 # Wait for incoming messages
#                 try:
#                     data = await websocket.receive_json()
#                     data_from_api = Input_FE(**data)

#                     objective=data_from_api.objective
                    
#                     create_survey=True

#                     if create_survey == True:
#                         response = recommend_sections(openai_model, objective)
#                         section_recommendations = response["recommend_sections_schema"]
                        
#                         # Get the existing complete_survey_data ONCE before the loop
                        
#                         # Set up the complete_survey_data structure ONCE before the loop
#                         complete_survey_data = {
#                             "blocks": [],
#                             "annotated_data": []
#                         }
                        
                        
#                         overall_section_index = len(complete_survey_data["blocks"])
                        
#                         # Refresh questions_list from the initial complete_survey_data
                        
#                         # Process each section
#                         for section_idx, section in enumerate(section_recommendations):
#                             section_name = section["section_name"]
#                             question_count = section["number_of_questions"]
#                             section_details = section["details_about_section"]
                            
#                             response = add_section(openai_model, section_details)
#                             section_name = response.get("section_name")
#                             number_of_questions = response.get("number_of_questions")
#                             text = response.get("details_about_section")
                            
#                             request = {"blocks": [{"type": section_name, "number_of_questions": number_of_questions, "text": text}]}
                            
#                             # Process each block for this section
#                             all_blocks = []
#                             annotated_data_entries = []
                            
#                             for block_index, block in enumerate(request.get("blocks")):
#                                 block_type = block.get("type")
#                                 objective = block.get("text")
#                                 number_of_questions = block.get("number_of_questions")
                                
#                                 payload = {
#                                     "action": "SHOW_LOADER",
#                                     "loader_text": f"Now generating: {section_name}. This may take a few moments."
#                                 }
#                                 await websocket.send_json(payload)
#                                 await asyncio.sleep(0.2)

#                                 # Create a block-specific questions dictionary
#                                 block_questions = {}
#                                 question_index = 1  # Reset for each block
                                
#                                 # Generate questions based on block type using the current questions_list
#                                 result = survey_gen.generate_raw_data_for_questions(
#                                     objective=objective,
#                                     number_of_questions=number_of_questions,
#                                     details=objective, 
#                                 )
                                
#                                 if "questions" in result:
#                                     for question in result["questions"]:
#                                         key = f"{question_index}_d"
#                                         block_questions[key] = question
#                                         question_index += 1
                                
#                                 # Process the block questions
#                                 processed_block_questions = await process_questions_parallel(
#                                     SurveyGen,
#                                     block_questions,
#                                 )
                                
#                                 # Update block_questions with processed results
#                                 block_questions.update(processed_block_questions)
                                
#                                 # Transform block questions to structured data
#                                 processed_blocks = process_new_format_data(block_questions)
                                
#                                 # Use the existing transformation with only this block
#                                 transformed_block_data = transform_new_survey_data(processed_blocks)
                                
#                                 # If there's any data from transformation, update the name property
#                                 if transformed_block_data and len(transformed_block_data) > 0:
#                                     for block_data in transformed_block_data:
#                                         # Set block name to the current block type
#                                         block_data["name"] = block_type
                                        
#                                         # Calculate the block index based on existing blocks plus current section
#                                         current_block_index = overall_section_index + section_idx
#                                         block_data["label"] = chr(65 + current_block_index)  # A, B, C, ...
                                    
#                                     # Add the transformed data to our blocks list
#                                     all_blocks.extend(transformed_block_data)
                            
#                             # Extract annotated data for this section
#                             for block in all_blocks:
#                                 if "questions" in block:
#                                     for question in block["questions"]:
#                                         if "annotated_text" in question:
#                                             # Create properly structured annotated_data entry
#                                             annotated_entry = {
#                                                 "id": question["id"],
#                                                 "nodes": question["annotated_text"]["nodes"],
#                                                 "questionType": question["annotated_text"]["questionType"]
#                                             }
#                                             annotated_data_entries.append(annotated_entry)
                            
#                             # Add this section's blocks to the complete survey data
#                             complete_survey_data["blocks"].extend(all_blocks)
#                             complete_survey_data["annotated_data"].extend(annotated_data_entries)
                            
#                             # Create the payload for this section
#                         payload = {
#                             "meta": {
#                                 "action": "FINAL_SURVEY"    
#                             },
#                             "complete_survey_data": complete_survey_data
#                         }
                        
#                         # Add the complete_survey_data to the payload
#                         payload["complete_survey_data"] = complete_survey_data
                        
#                         # Send the response for this section
#                         await websocket.send_json(payload)

#                         # try:
#                         #     await websocket.close()
#                         # except Exception as e:
#                         #     pass
                        
                    
                
#                 except Exception as e:
#                     exception_name = type(e).__name__
#                     if any(name in exception_name for name in ["WebSocketDisconnect", "ConnectionClosed", "WebSocketError"]):
#                         print(f"Client disconnected: {str(e)}")
#                         break
#                     elif "ValueError" in exception_name:
#                         # Handle JSON parsing errors
#                         print(f"Invalid JSON received: {str(e)}")
#                         try:
#                             await websocket.send_json({"error": "Invalid data format"})
#                         except Exception:
#                             # If sending fails, client likely disconnected
#                             print("Failed to send error response, client likely disconnected")
#                             break
#                     else:
#                         # Handle other errors during processing
#                         print(f"Error processing request: {str(e)}")
#                         try:
#                             await websocket.send_json({"error": str(e)})
#                         except Exception:
#                             # If sending fails, client likely disconnected
#                             print("Failed to send error response, client likely disconnected")
#                             break
            
#         except Exception as e:
#         # This catches any exceptions not caught in the inner try-except
#             print(f"Outer exception in WebSocket handler: {str(e)}")
        
#         finally:
#             # Always execute this when the loop exits for any reason
#             print("WebSocket handler exiting, cleaning up...")
#             try:
#                 # Check if we can access the client_state property
#                 if hasattr(websocket, "client_state"):
#                     # Check if we should compare against a constant
#                     if hasattr(WebSocketState, "DISCONNECTED"):
#                         if websocket.client_state != WebSocketState.DISCONNECTED:
#                             await websocket.close(code=1000)
#                     # Otherwise use a string or other comparison
#                     else:
#                         if getattr(websocket.client_state, "DISCONNECTED", None) is not True:
#                             await websocket.close(code=1000)
#                 else:
#                     # Fallback - try to close anyway with exception handling
#                     try:
#                         await websocket.close(code=1000)
#                     except Exception:
#                         pass
#                 print("WebSocket connection closed gracefully")
#             except Exception as e:
#                 print(f"Error during WebSocket cleanup: {str(e)}")


#     return fastapi_app





from typing import Optional
import json
import modal
import pydantic  # for typing, used later
from typing import Optional, Dict, List, Union, Any
import uuid
import os
from fastapi import FastAPI, WebSocket
import random
import string
import re
from typing import Dict, Any
import asyncio
from pydantic import BaseModel, Field
import time
import httpx
from fastapi import Request, HTTPException
from openai import OpenAI
from Functions.helper import log_time
from Functions.process_single_question_wrapper import *
from Question_TypesV2.number_of_question import *
from Question_TypesV2.split_questions import split_questions
import httpx
from Functions.images import Input_FE
from asyncio import to_thread
from concurrent.futures import ProcessPoolExecutor
from Functions.images import *
from Functions.check_user_status import check_user_status
from Functions.accept_suggestion import accept_suggestion
from typing import List, Optional, Literal
import asyncio
import time
import pydantic
from modal import web_endpoint, Secret, app
from fastapi import Request
from Functions.router import router_prompt
from Cursor.general_chat import general_chat
from Cursor.add_question import add_question
from Cursor.add_section import add_section
from Cursor.delete_a_question import delete_question
from Cursor.find_question_text_from_label import find_question_text_by_label
from CreateSection.Functions.annotation import annotation
from CreateSection.Functions.process_questions_in_parallel import process_questions_parallel
from CreateSection.Functions.transform_survey_data import *
from CreateSection.Functions.image import survey_generator
from CreateSection.Functions.identify_questions import SurveyGen
from CreateSection.Functions.identify_questions import *
from Cursor.generate_new_label import generate_next_question_label
from Cursor.delete_question_from_payload import delete_question_by_label
from CreateSection.Functions.extract_survey_questions import extract_survey_questions
from Edit_Selection.prompts.correction import correction_prompt
from CreateSection.Functions.get_block_names import get_block_names
from Functions.get_complete_questions import generate_complete_questions
from Cursor.finalize_objective import finalize_survey_objective
from Cursor.objective import determine_objective
from Functions.create_conversation import extract_conversation_from_metadata
from Cursor.recommend_sections import recommend_sections
from Functions.extract_question_labels import extract_question_labels
from Cursor.edit_from_chat import correction_prompt_via_chat
from Functions.get_question_data_from_label import find_question_by_label
import requests

from Functions.recommend_provider_segments import recommend_provider_segments
import websockets
from starlette.websockets import WebSocketState
from Cursor.budget_details import budget_details
from Functions.router_publshed import router_prompt_published
from Functions.extract_block_names import extract_block_names
from Functions.delete_sections import delete_sections
from Functions.optimize_survey_instructions import optimize_survey_instructions
from Functions.get_complete_payload import generate_complete_questions_payload
from Cursor.recommed_sections_to_optimize import recommend_sections_to_optimize
from Cursor.add_section_for_optimize import add_section_for_optimize
from Functions.estimate_time import estimate_time
from Functions.request_bid import request_bid
from Cursor.validate import validate_survey
from Functions.get_raw_data import get_raw_data
from Cursor.fix_post_validation import fix_post_validation
from Functions.finalize_bid import finalize_bid
from Functions.extract_first_section import extract_first_section
from Cursor.quota import quota_creator

app = modal.App("Create Survey")


with open("CreateSection/Functions/screener_section.json", "r") as f:
    readymade_screener_section = json.load(f)

@app.cls(
    image=categorizer,
    secrets=[modal.Secret.from_name("openai-secret")]
    # enable_memory_snapshot=True
)
class OpenAIModel:

    def __init__(self):
        self.client_open_ai = OpenAI()
 
        



@app.function(image=testing_image, secrets=[modal.Secret.from_name("openai-secret")], timeout=2000, min_containers=1)
@modal.web_endpoint(method="POST")
async def create_survey(data: Input_FE):
    fastapi_app = FastAPI()
    client = OpenAI()
    survey_gen = SurveyGen()
    objective = data.objective
    openai_model = OpenAIModel()

    response = recommend_sections(openai_model, objective)
    section_recommendations = response["recommend_sections_schema"]
    
    # Get the existing complete_survey_data ONCE before the loop
    
    # Set up the complete_survey_data structure ONCE before the loop
    complete_survey_data = {
    "blocks": [],
    "annotated_data": []
}


    overall_section_index = len(complete_survey_data["blocks"])

    # Refresh questions_list from the initial complete_survey_data

    # Process each section
    for section_idx, section in enumerate(section_recommendations):
        section_name = section["section_name"]
        question_count = section["number_of_questions"]
        section_details = section["details_about_section"]
        
        response = add_section(openai_model, section_details)
        section_name = response.get("section_name")
        number_of_questions = response.get("number_of_questions")
        text = response.get("details_about_section")
        
        request = {"blocks": [{"type": section_name, "number_of_questions": number_of_questions, "text": text}]}
        
        # Process each block for this section
        all_blocks = []
        annotated_data_entries = []
        
        for block_index, block in enumerate(request.get("blocks")):
            block_type = block.get("type")
            objective = block.get("text")
            number_of_questions = block.get("number_of_questions")
            
            payload = {
                "action": "SHOW_LOADER",
                "loader_text": f"Now generating: {section_name}. This may take a few moments."
            }
            

            # Create a block-specific questions dictionary
            block_questions = {}
            question_index = 1  # Reset for each block
            
            # Generate questions based on block type using the current questions_list
            result = survey_gen.generate_raw_data_for_questions(
                objective=objective,
                number_of_questions=number_of_questions,
                details=objective, 
            )
            
            if "questions" in result:
                for question in result["questions"]:
                    key = f"{question_index}_d"
                    block_questions[key] = question
                    question_index += 1
            
            # Process the block questions
            processed_block_questions =  await process_questions_parallel(
                SurveyGen,
                block_questions,
            )
            
            # Update block_questions with processed results
            block_questions.update(processed_block_questions)
            
            # Transform block questions to structured data
            processed_blocks = process_new_format_data(block_questions)
            
            # Use the existing transformation with only this block
            transformed_block_data = transform_new_survey_data(processed_blocks)
            
            # If there's any data from transformation, update the name property
            if transformed_block_data and len(transformed_block_data) > 0:
                for block_data in transformed_block_data:
                    # Set block name to the current block type
                    block_data["name"] = block_type
                    
                    # Calculate the block index based on existing blocks plus current section
                    current_block_index = overall_section_index + section_idx
                    block_data["label"] = chr(65 + current_block_index)  # A, B, C, ...
                
                # Add the transformed data to our blocks list
                all_blocks.extend(transformed_block_data)
        
        # Extract annotated data for this section
        for block in all_blocks:
            if "questions" in block:
                for question in block["questions"]:
                    if "annotated_text" in question:
                        # Create properly structured annotated_data entry
                        annotated_entry = {
                            "id": question["id"],
                            "nodes": question["annotated_text"]["nodes"],
                            "questionType": question["annotated_text"]["questionType"]
                        }
                        annotated_data_entries.append(annotated_entry)
        
        # Add this section's blocks to the complete survey data
        complete_survey_data["blocks"].extend(all_blocks)
        complete_survey_data["annotated_data"].extend(annotated_data_entries)
        
        # Create the payload for this section
    payload = {
        "meta": {
            "action": "FINAL_SURVEY"    
        },
        "complete_survey_data": complete_survey_data
    }

    # Add the complete_survey_data to the payload
    payload["complete_survey_data"] = complete_survey_data

    return payload




import json
from utils import retry_on_json_error

@retry_on_json_error
def initiate_survey(openai_model, prompt):
        
    output_schema = { "questions_to_add": [
        {
        "response": "string",
        "question_text": "string" # (markdown formatted)
        },
        {
        "response": "string",
        "question_text": "string" # (markdown formatted)
        }, 
    
    ]}
    parsed_text = openai_model.client_open_ai.chat.completions.create(
        model="gpt-4o",
        temperature=0.0000000001,
        response_format={ "type": "json_object" },
        messages=[
                {"role": "system", "content": "You are a helpful assistant that can add questions to a survey."},
                {"role": "user", "content": f"""
You are a helpful text editor chatbot for Visceral, a cutting-edge AI-native platform that modernizes market research by automating survey programming, quality testing, and analytics. Visceral integrates with major panel companies, data collection platforms, and GenAI systems, enabling surveys to run 10x faster at 50% cost while maintaining high standards for market researchers. It’s an open, A-to-Z platform with drag-and-drop and API capabilities, designed for rapid adoption without disrupting existing workflows or vendors.
Your role is to assist users in adding a screener section to a survey. please give me an introductory line 
please give me a valid response in a json format.
the user's message is: {prompt}
follow this output schema strictly:
{json.dumps(output_schema)}


                """
                }
            ]
        )

    content = parsed_text.choices[0].message.content
    result = json.loads(content)
    return result





import json
from utils import retry_on_json_error


@retry_on_json_error
def unknown_question(openai_model, prompt):
        schema = """
        {
            "ask_logic": "", # Capture any "ask_if" logic that specifies when a question should be asked.
            "questionLabel": "string",  # Capture the label exactly as it appears, even is there's any text till we start the options, should be captured as it is. we dont want to lose any text at all, not even a single dot or bracket.
            "question": "string",  # Capture the full question text as it is.
            "type"="unknown"
        }
        """

        parsed_text = openai_model.client_open_ai.chat.completions.create(
            model="gpt-4o",
            temperature=0.0000000001,
            response_format={ "type": "json_object" },
            messages=[
                {"role": "system", "content": "You are a precise survey question parser that captures every detail without losing any information. The most important thing is that we should not delete even a single dot. You are dealing with unknown question type"},
                {"role": "user", "content": f"""
                You are a detailed survey question parser. Your task is to analyze survey questions and capture EVERY piece of text while structuring them into a specific JSON format. Follow these guidelines:

                CORE PRINCIPLES:
                1. Never delete any text. WE SHOULD NOT LOSE EVEN A SINGLE DOT. we cannot delete any text at any cost.
                 
                The schema to be used is : {schema}


                PARSING RULES:

                1. Question Label:
                - Extract the exact identifier at the start (e.g., "Q1.", "5Kn.", etc.)
                - Keep it exactly as written

                2. Question Text:
                - Include everything from the start of the question to the end.
                
        
                
                SPECIAL INSTRUCTIONS:
                
                - Keep all original spacing and formatting
                - Never remove any text, even if it seems redundant, we should be capturing each and every detail. we should not delete even a single dot.
                


                Examples:

                Input:
                ```
                Ask if Q3==0 \n QM. Demo unknown Question? Disqualify if QM=="XYZ" \n random text
                ```

                Output:
                {{
                    "questionLabel": "QM",
                    "question": " Ask if Q3==0 \n QM. Demo unknown Question? Disqualify if QM=="XYZ" \n random text",
                    "type":"unknown",
                 
                }}
                
          
                CORE PRINCIPLES and MOST IMPORTANT:

                Never delete any text: Every part of the input must be accounted for. Missing even one word is a critical error.
                The most important thing is that we should not lose any text!!
                Important: please give me a valid json.
                Parse the following survey question according to these rules: 
                 
                {prompt}
                """
                }
            ]
        )
       
        content = parsed_text.choices[0].message.content
        result = json.loads(content)
        return result





import json
import asyncio
from utils import retry_on_json_error

@retry_on_json_error
async def annotation(openai_model,prompt,keymap, type_of_question,user_tags, variable_list_keyword=None):
    
    template="""
    {
    "nodes": [
        {
            "type": "",
            "text": "",
            "data": {  // Include this field only if the type is "logic"
                "subtype": ""  
            }
        }
    ],
    "questionType": "questionType"
}

"""
    print("the variable keyword is :", variable_list_keyword)

    parsed_text = await asyncio.get_event_loop().run_in_executor(
        None,
        lambda: openai_model.client_open_ai.chat.completions.create(
        model="gpt-4o",
        # model="o1",
        temperature=0.0000000001,
        response_format={ "type": "json_object" },
        messages=[
            {"role": "system", "content": "You are a precise survey question parser that captures every detail without losing any information."},
            {"role": "user", "content": f"""
            

                You are a precise survey question annotator. Your task is to tag all parts of a survey question into a JSON structure following a node-based annotation system. 
                The most important rule is to NEVER delete, modify, or rearrange any part of the input text. Each part of the text must be categorized appropriately without loss.
                line breaks should be explicitly captured using {{type: “linebreak”}}. Important: Line breaks in the input will be explicitly marked as <lbr />
                
                We currently support the following logics: randomize, skips, terminate, range, ask_if, piping. (these will be the values of the subtype in the data)
                If you encounter any of these logics, ensure they are captured under the logic type.
                There may be multiple ways to write a logic. For example:
                Variants such as "End," "Route to terminate," and "Terminate" should all be categorized under Terminate.
             
                Additionally: 
                If a user explicitly tags a logic, it will appear in the input like:
                <logic type="disqualify"> demo logic text here </logic>
                In this case, return the node as:
                {{type: "logic", text: "demo logic text here", data: {{subtype: "terminate"}} }}
             
             
                Make sure to add the question type in the JSON under a field called questionType, this question has been catrgorized as : {type_of_question}

                Annotation Guidelines:
                - Use the following node types:
                  - {{type: “label”, text: “S5_Q20”}}: Represents a label for the question. (The label is the unique identifier given before the question starts)
                  - {{type: “question_text”, text: “Demo question main text?”}}:  The main text of the question shown to the survey taker. remember that we dont want to show any programmer notes or notes to the user in the question_text, it should only have the main question text which will be shown to the user taking the survey.
                  - {{type: “linebreak”}}: Indicates a line break character in the input. Line breaks in the input will be explicitly marked as <lbr />
                  - {{type: “not_displayed”, text: “any other random text”}}: Any text we wont be showing it to the person taking the survey. 
                  - {{type: “unknown”, text: “any unclear or ambiguous text”}}: For anything that cannot be categorized. For unclear or ambiguous text that cannot be categorized. This will appear as a red line to the user, indicating that the input is not understood.
                  - {{type: “logic”, text: “Any text that shows a logic”, data: {{subtype: ""}} }}:  Represents logical conditions or rules. Include a data field for subtypes. (The subtype can be  : Randomize, Skips, Terminate, Range, Ask if, Piping. )
                  - {{type: "option_label", text: "Any unique identifier}}
                  - {{type: “options”, text: “India”}}: For selectable options in the survey.
                  - {{type: "variable", text:""}} : A user-defined list of variables (e.g., [{keymap}]) will be provided. if blank, ignore this. But If a variable from the list appears in the input text, capture it.
                {{type: "questionType","text":""}} : the questionType for this question is : {type_of_question}
             
                Each question must have : "label" and "question_text". Always ensure that the label is short and appears before the question starts. like "Qm, Q8, Qnkl." something like this. 
                Important: A question label must be a unique identifier and cannot consist of more than one word. Examples include "Q1.", "5Kn.", etc. Pay close attention to this requirement.
                A label can be followed by instructions and then the question_text. Be thoughtful and precise when choosing the label.
                The “label” may be followed by an instruction, so exercise judgment when determining this.
                Keep in mind that a label cannot be a complete sentence; it is just a concise identifier that precedes the question text.
                - Do not interpret or deduce meaning. Just tag text based on the input.
                - Important: Explicitly tag any <lbr /> in the input as {{type: "linebreak"}}.
                - Preserve all formatting from the input.
                - backslash tab indicates spacing, so make sure to capture the spacing properly, we should not lose even a space.
                - the incoming text can be in any language so please make sure to capture everything accordingly.

                Example Input:
                S5_Q20 <lbr />
                Demo Question Main text? <lbr />
                Any terminate logical statement  for example<lbr />
                op1, op2, op3<lbr />
                random unclear text that we cant understand.

                Example Output:
                {{
                "nodes": [
                {{
                  {{type: "label", text: "S5_Q20 "}},
                  {{type: “linebreak”}}
                  {{type: "question_text", text: "Demo Question Main text? "}},
                  {{type: “linebreak”}}
                  {{type: "logic", text: "Any terminate logical statement  for example", "data" : {{"subtype":"terminate"}}}},
                  {{type: “linebreak”}}
                  {{type: "options", text: "op1"}}
                  {{type: “not_displayed”, text: “, ”}}
                  {{type: "options", text: "op2}}
                  {{type: “not_displayed”, text: “, ”}}
                  {{type: "options", text: "op3}}
                  {{type: “linebreak”}}
                  {{type: “unknown”, text: “random unclear text that we cant understand.”}}
                }}
                ],
                "questionType":"single_select"
                }}

                please follow this template to structure your output: {template}
                Most important: 
                1. We cannot lose even a single dot, each and every space should be accounted for, you cannot move even a single character. it should be where it was.                 
                2. Preserve all formatting from the input.
                3. you are not supposed to add anything or modify the text. 
                4. Line breaks will only be there is explicitly there in the text
                5. Make sure to correctly capture the question_text. understand what will be shown to the user as they will be taking that survey question.
                
                you are not supposed to add anything or modify the text. 

                {'Important: The user has explicitly mentioned some things that should be captured: ' + user_tags if user_tags else ''}
                
                {"Important, The user explicitly mentioned to use " + variable_list_keyword + " in the input text to be marked as options node" if variable_list_keyword else ""}
                
                Please give me a valid json
                
                the input text for which you have to create annotations is  : {prompt}
                
                

            """
            }
        ]
    ) )

    content = parsed_text.choices[0].message.content
    result = json.loads(content)
    return result











import json
import asyncio
from utils import retry_on_json_error

@retry_on_json_error
async def recommend_provider_segments(openai_model,audience,budget):
    
    

    parsed_text = openai_model.client_open_ai.chat.completions.create(
        model="gpt-4o",
        # model="o1",
        temperature=0.0000000001,
        response_format={ "type": "json_object" },
        messages=[
            {"role": "system", "content": "You are a panel recommendation expert. You will be receiving a audience and you need to return the correct provder it will fall into."},
            {"role": "user", "content": f"""
            You are a panel recommendation expert. You will be receiving a audience and you need to return the correct provder it will fall into.
            We currently support the following providers:  Consumer, B2B.
            you will also be receiving the budget of the user, please make sure you return me the exact budget but without any dollar sign or commas.
            
            Please return the json in the following format:
            {{
                "provider": "provider",
                "budget": "budget"
            }}
            please give me a valid json.
            make sure you use the exact names for the providers as mentioned above.
            please mark the audience, with the correct provider. The audience is: {audience} 
            and the budget is: {budget}

            """
            }
        ]
    ) 

    content = parsed_text.choices[0].message.content
    result = json.loads(content)
    return result






def extract_first_section(test_payload):
    print("test_payload", test_payload)
    """
    Extracts questions from the first block/section of the survey data.
    
    Args:
        test_payload: The survey data payload
        
    Returns:
        List of questions from the first block, or empty list if not found
    """
    try:
        # Access blocks - try both attribute and dictionary access
        if hasattr(test_payload, 'blocks'):
            blocks = test_payload.blocks
        elif isinstance(test_payload, dict) and 'blocks' in test_payload:
            blocks = test_payload['blocks']
        else:
            print("No blocks found in payload")
            return []
        
        # Check if blocks is empty
        if not blocks or len(blocks) == 0:
            print("Blocks list is empty")
            return []
        
        # Get the first block
        first_block = blocks[0]
        
        # Get questions from the first block
        if isinstance(first_block, dict) and 'questions' in first_block:
            return first_block['questions']
        elif hasattr(first_block, 'questions'):
            return first_block.questions
        else:
            print("No questions found in the first block")
            return []
            
    except Exception as e:
        print(f"Error extracting first section: {e}")
        return []


import json
import asyncio

def format_text(openai_model, prompt):
    parsed_text = openai_model.client_open_ai.chat.completions.create(
            model="gpt-4o",
            temperature=0.0000000001,
            response_format={"type": "json_object"},
            messages=[
                {"role": "system", "content": "You are a precise survey question parser that captures every detail without losing any information."},
                {"role": "user", "content": f"""
                You will receive an input text that contains a lot of noise. Your task is to format it properly using Markdown while ensuring that no characters (such as dots or punctuation) are lost. However, you may adjust line breaks to improve readability.
                Remember, we cannot lose any text. It's like we are copy pasting stuff but just formatting it for proper readability.
                You are not supposed to move any text, keep it whereever it was.
                please donot add anything, you just have to format it.

                Return the output in the following JSON format:
                {{
                    "mdText": "<formatted_markdown_text>"
                }}

                Input text:  
                {prompt}
"""}
            ]
        
    )

    content = parsed_text.choices[0].message.content
    result = json.loads(content)

    return result



import json

def multi_select_question(openai_model, prompt,keymap_name):
        schema = """
        {
            "ask_logic": "", # Capture any "ask_if" logic that specifies when a question should be asked.
            "questionLabel": "string",  # Capture the label exactly as it appears, even is there's any text till we start the options, should be captured as it is. we dont want to lose any text at all, not even a single dot or bracket.
            "question": "string",  # Full question text with any logic in `{ logic: None }` format . YOU ARE NOT SUPPOSED TO DELETE ANYTHING COPY IT AS IT IS. 
            "options": [  # List of options (if any) with their associated logic if any
                {"full_option_text": "None"}
            ],
            "extra_text": "string", # Any text that does not fit into the specified categories should be captured here. The purpose of "extra_text" is to ensure no data is lost, allowing us to retain every detail from the input text, including any text or symbols that fall outside the defined structure. (if there's any logic, then make sure to wrap it in `{logic: None}`)
            "type"="multi_select",
            "keymap": "string"
            
        }
        """

        parsed_text = openai_model.client_open_ai.chat.completions.create(
            model="gpt-4o",
            temperature=0.0000000001,
            response_format={ "type": "json_object" },
            messages=[
                {"role": "system", "content": "You are a precise survey question parser that captures every detail without losing any information. The most important thing is that we should not delete even a single dot. You are dealing with multi_select question type"},
                {"role": "user", "content": f"""
                You are a detailed survey question parser. Your task is to analyze survey questions and capture EVERY piece of text while structuring them into a specific JSON format. Follow these guidelines:

                CORE PRINCIPLES:
                1. Never delete or modify any text. WE SHOULD NOT LOSE EVEN A SINGLE DOT. we cannot delete any text at any cost.
                 
                The schema is : {schema}
                

                PARSING RULES:

                1. Question Label:
                - Extract the exact identifier at the start (e.g., "Q1.", "5Kn.", etc.)
                - Keep it exactly as written

                2. Question Text:
                - Include everything from the start of the question to just before the options start. anything and everything, even if it doent make sense will be a part of the question text.
                - Even random or unrelated text must be part of the question.
                - For any logic/instructions in parentheses or brackets, wrap them in backticks like `{{"text": None}}` format
                - Example: "How old are you? (Disqualify if <18)" becomes:
                "How old are you? `{{"Disqualify if <18": None}}`"

                3. Options:
                - Be smart enough to understand the options from the text.
                - Create a list of dictionaries where each option is a key and value as None
                - example {{"Option text": None}}
                - example {{"Option text (Skip to Q4)": None}}
                - example: {{"Option text Terminate": None}}
                Important: If options have numbers in front of them, use the order the numbers indicate:

                For 04. a 03. b 02. c 01. d, capture the order as (d, c, b, a) (descending).
                For 01. a 02. b 03. c 04. d, capture the order as (a, b, c, d) (ascending).

                Make sure to follow the order as mentioned in the input text if it has the numbering in front of it.
                

                4. Text After Options:
                - Capture any text that appears after the options list
                - Wrap any logic in `{{"text": None}}` format
                - Important: Text after options will exist even if there are no options, it there to capture the ending text. so that we dont lose any text.
    
                 
                5.Keymap: Identify and capture any exact matches of keymap variable names when they appear in the user's input text. These variables serve as predefined references within our platform.
                Important: you dont have to mention the keymap in the question text then.
                
                A keymap will always be from {keymap_name if keymap_name is not None else ": User doesnt have any keymaps defined"}
                If the user explicitly mentions the variable, it should be recognized and captured accurately.
                This behavior is distinct from "piping" and specifically focuses on detecting variable names.
                Some of the user defined key map names are : {keymap_name} and you dont have to add that variable separately in the options if given additional options.
                
                
                Important: Capture any ask if logic in the ask_logic variable. By 'ask if' logic, I mean any statement that determines when a question should be asked.
                
                SPECIAL INSTRUCTIONS:
                
                - All logic must be wrapped in backticks `{{}}` 
                - Important : The logics that we support are :  dropdown, randomize, terminate, skips. (It is possible that they can be written in some other way, for example : end also means termination) Be smart to identify.
                - Keep all original spacing and formatting
                - Never remove any text, even if it seems redundant, we should be capturing each and every detail. we should not delete even a single dot.
                

                Important: if you come across any question, which asks to fill it's options from previous question it means it is a a piping question. then please add a field piping with a dictionary where the  key is the questionLabel from which we have to fetch responses. and value could be any from these 4: [show selected, show not selected, hide selected, hide not selected].
                for example . you see something like this in a text: "Show all selected in QM. " then your json will have a field like : piping : {{"QM":"show selected" }}
                Important:  piping is used when the text clearly states to fill options from previous question and the question label is not in {keymap_name} if you see {keymap_name} in the text then in clearly means that it is keymap and not piping.
                Examples:

                Input:
                ``` Ask Qm if QN==01 \n
                QM. Demo Question (Disqualify if <1) Randomize \n random text
                1. option a asjkdn
                2. option b Skip to Q3
                3. option c Terminate
                Must select one option
                ```
                for the option you dont have to use backticks.it will be dictionary.
                Output:
                {{
                    "ask_logic"="Ask Qm if QN==01",
                    "questionLabel": "QM",
                    "question": "Demo Question? `{{"Disqualify if <1": None}}` `{{"Randomize": None}}` \n random text",
                    "options": [
                        {{"option a asjkdn": None}},
                        {{"option b Skip to Q3": None}},
                        {{"option c Terminate": None}}
                    ],
                    "extra_text": "Must select one option",
                    "type":"multi_select",
                }}
                


                If a question references a previous one and requests to "pipe in" or "port in" results, follow these steps:
                Identify the referenced question. For example, if the prompt says:
                "Q3. Why do you like (port in response from Q2)",
                replace the placeholder with {{{{Q2}}}}.
                The question should then look like:
                "Why do you like {{{{Q2}}}}".
                Ensure the label (e.g., Q2) matches the format of the referenced question.
                If the reference uses terms like "Recall #2" or similar, replace it with the corresponding question label in the format {{{{Q2}}}} (not {{{{Recall #2}}}}).
                Use the exact question label, such as Q2, based on how it is defined in the context.
                Do not include any extra text or terms from the reference; only use the question label.
                 
                Important: the spellings can we wrong, but you have to see the bigger picture of what the user is trying to say or what kind of logic is the user trying to write.
                Important: there can be multiple ways of writing a logic, you have to understand if it means terminate, skip to.. , randomize or if the user wants to show the question as dropdown. 
                Important: the text of the options will have the complete text and the value of the options will always be None. in the options you dont need to capture any logic separately. just capture the entire text as it is without deleting anything. we should not delete anything.
                Important : the value in the options will always be None
                Important: a question will be until the options start. so if there's anything till the options start should be captured in the question.
                
                CORE PRINCIPLES and MOST IMPORTANT:
                
                
                Never delete any text: Every part of the input must be accounted for. Missing even one word is a critical error.
                The most important thing is that we should not lose any text!!
                Important: please give me a valid json.
                you just have to wrap the logic in `{{}}` , if you are confused about any text if it is a logic or not, then you can just keep it in the text without wwapping it in `{{}}`
                important: if there's any logic you dont understand, just keep it as a part of the question text.
                Finally, ensure that every piece of text from the input has been captured in the JSON without any loss. If any part of the text does not fit into the defined schema, it should be included in the extra_text field. The goal is to retain all input text in its entirety.
                Parse the following survey question according to these rules: {prompt}
                 
                """
                }
            ]
        )

        content = parsed_text.choices[0].message.content
        result = json.loads(content)
        return result



def multi_select_logic(openai_model, input_json,user_logic):
    

    parsed_text = openai_model.client_open_ai.chat.completions.create(
        model="gpt-4o",
        temperature=0.0000000001,
        response_format={ "type": "json_object" },
        messages=[
            {"role": "system", "content": "You are a precise survey question parser that captures every detail without losing any information."},
            {"role": "user", "content": f"""
            



            You will be receving a json which has single_select question , your task is to understand any logics if they exist and put it in the bucket of logics we currently support.
            
            for the options  : please update the values with the appropriate logic
            
            We currently support :
            Any termination logic should be marked as : "THANK AND END" 
            Any Skip logic should be marked as : "SKIP TO [qstnLabel]" where qstnLabel is the question label
            Any logic which EXPLICITLY SAYS to finish if not selected then it should be marked as : "NOT SELECTED". for example : Skip to Term if not Q2=2  (if it is explicitly mentioned only then )
            Any option which has "none of the above" : "nota" 
            any option which has "specify" or any option which says "please specify" :  "other" 
            if an option has (prefer not to answer ) : "optout"
            if an option has "not applicable": "notapplicable"
            None (if no logic associated) with the option.
            
            Specify is not a logic , so dont wrap it up in backticks
             
            If the extra_text contains logic statements like: "'Disqualify IF Q3 == `01` Disqualify IF Q2 == `02` Disqualify IF Q2 == `04`'" Then follow these rules: First, check if the QuestionLabel matches the one mentioned in the logic (e.g., for Disqualify IF Q2 == '02', the QuestionLabel must be Q2). If it matches, apply the logic only to the specific option mentioned. For instance, if Q2 has an option 02, assign "THANK AND END" to this option, you donot have to add anything to the key, but just the values., not to others. Similarly, handle "Skip to..." logic in the same way—apply it only to the specified option and QuestionLabel.
            for example:
            {{'ask_logic': '', 'questionLabel': 'Q1.', 'question': 'What do you like?', 'options': [{{'Apple': None}}, {{'Pineapple': None}}, {{'Mango': None}}, {{'Kiwi': None}}], 'extra_text': 'Disqualify IF Q3 == `01` Disqualify IF Q1 == `02` Disqualify IF Q1 == `04`', 'type': 'single_select'}} and we can see extra_text has "Disqualify IF Q3 == `01` Disqualify IF Q1 == `02` Disqualify IF Q1 == `04`" which menas Q3 is not equal to Q1(questionlabel ) so we ignore that, but for others it matches hence Disqualify IF Q1 == `02` Disqualify IF Q1 == `04`, in fornt of them we will add "THANK AND END" so the output would be : 
            {{'ask_logic': '', 'questionLabel': 'Q1.', 'question': 'What do you like?', 'options': [{{'Apple': None}}, {{'Pineapple': 'THANK AND END'}}, {{'Mango': None}}, {{'Kiwi': 'THANK AND END'}}], 'extra_text': 'Disqualify IF Q3 == `01` Disqualify IF Q1 == `02` Disqualify IF Q1 == `04`', 'type': 'single_select'}}
           
             

             important : you will have to update the key and value of the options dictionary 
            for exmaple: if you received this as an option text :  {{'None allenSir': None}} and we know that the user writes allennSir as Disqualify then the option text in the new json will be {{None `allenSir`: "Terminate"}} this means that in the new json we will wrap that particular logic in backticks and in the value of the dictionary we will put the logic from our bucket. Donot delete any text.
            in the key, make sure to put the logic in the backticks, even if the logic is in brackets. put the enitre logic incuding any brackets etc. in the backticks.
            
            important: if you see anything related to randomize. please create a new field called {{"randomize":True}} 
            important: if you see anything related to dropdown, please create a new field called {{"dropdown":True}}
            important: if you see anything related to ranges, please create a new field called {{"data_range":[min,max]}}
             

            also, please update the dictionary to True where the logic was intially mentioned.
            for example : input: {{"question" : "demo question `{{Randomize:None}}`"}}
            then you will give me : {{"question":"demo question `{{Randomize:True}}`", "randomize":True }}, so basically we created a new field and also updated the logic in the question to True.
                
            
            be sure to smartly understand what the user is trying to mean with that option. There can be multiple ways to say all these logics.
            Important : Apart from that: Ill be sharing with you the users way of writing a logic for the survey. So you should be smart enough to understand what the user is trying to say and then make our json.
            Important : The users way of writing the logic is : {user_logic} make sure to only use the words as logics which are explicitly mentioned
            please pay special attention while assigning logics. 

            example input option: {{"pizza (loppy)":None}} and the user marks loppy as disqualify then your output will be : {{"pizza `(loppy)`:"THANK AND END"}}. in the key, you only have to wrap the logic in backticks if you can identify it. otherwise no need for example: {{"pizza (choppy)":None}} and we dont identify any logic here so we will not wrap the logic with backticks. the output will be  {{"pizza (choppy)":None}}
            
            For complete and disqualify -> we will use "THANK AND END" in our final json that we produce and for skip we use "SKIP TO [qstnLabel]" . 
            Be sure to understand how a user writes the logic. 
            Use you general understanding and the user_logic way of writing the logic. 
            Also, if the logic is not in english. please understand it properly
        
            Very Important: You are not supposed to add any new options on your own, just deal with whatever you have.
            
            for example if you get an input that has only one option like 'options': {{'una ..1': None}} then you are not supposed to add any new options. 
            Most Important: No data should be deletd, not even a single dot.
            Important : you are not supposed to change the logic field in the json.

            
            important: "Did not vote" or "None" ,"exclusive" anything like that doesnt always mean disqualified. until it is explicitly mentioned.
            Please pay special attention to the instructions for logics and make sure you mark it correctly, double check it. 
            Important: Donot even delete a single bracket.
            Most Important: No data should be deletd, not even a single dot.
            The json is :{input_json}      


                    
            please give me the correct json. 
            """
            }
        ]
    )

    content = parsed_text.choices[0].message.content
    result = json.loads(content)
    return result




from Functions.helper import clean_config_text, clean_option_text, format_label, map_config_ui, map_subtype, map_type, extract_logic_from_backticks
from Functions.process_logics import *
from Functions.nano_id import *
from Functions.maxdiff import *
import time
from Functions.dependencies import extract_dependencies
from Functions.process_text_for_ami_dir import process_text
def convert_lbr_to_newline(text):
    return text.replace('<lbr />', '\n')


def process_single_question(question_data: dict, raw_data:str, key_id=None, variable_list_keyword=None, use_ami=None, openai_model=None) -> dict:
    try:
        if not isinstance(question_data, dict):
            raise ValueError("Question data must be a dictionary")
        
        # Get basic question properties

        print("Value of use_ami: ",use_ami)

        if use_ami:
            label = question_data.get('Ref', '')
            if not label:
                label = format_label(question_data.get('questionLabel', '') or question_data.get('qstnLabel', ''))
        else:
            label = format_label(question_data.get('questionLabel', '') or question_data.get('qstnLabel', ''))



        # Don't clean the question text - preserve original
        # question_text, extracted_logic = extract_logic_from_backticks(question_data.get('question', ''), clean_text=True)
        print("the value of use_ami is ", use_ami)
        raw_question = question_data.get('question', '')

        # question_text_uncleaned = clean_question_text(raw_question)
        # question_text=clean_config_text(question_text_uncleaned)

        # question_text_uncleaned = clean_question_text(raw_question)
        question_text=clean_config_text(raw_question)

        
        question_type = question_data.get('type', '')
        if use_ami:
            question_text=process_text(openai_model, question_text, question_type)
        
        options = question_data.get('options', [])
        extra_text = question_data.get('extra_text', '')
        # extra_text=clean_question_text(text_after_options_uncleaned)
        dependencies = extract_dependencies(question_data)
        ask_logic = question_data.get('ask_logic', '')
        condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else None
        
        # Initialize processed question
        current_time = int(time.time() * 1000)
        has_options = bool(options)

        # Create base structure for extra_text
        base_structure = {
            "raw_data": raw_data,
            "dependencies":dependencies

        }

        if 'piping' in question_data and question_data['piping']:
            piping_config = {}
            
            for prev_question_label, pipe_type in question_data['piping'].items():
                current_question_label = label
                piping_filter = ""
                
                if pipe_type == 'show selected':
                    piping_filter = f"FILTER_X(OPTIONS({prev_question_label}), X IN {prev_question_label})"
                elif pipe_type == 'show not selected':
                    piping_filter = f"FILTER_X(OPTIONS({prev_question_label}), X NOT IN {prev_question_label})"
                elif pipe_type == 'hide selected':
                    piping_filter = f"FILTER_X(OPTIONS({current_question_label}), X IN {prev_question_label})"
                elif pipe_type == 'hide not selected':
                    piping_filter = f"FILTER_X(OPTIONS({current_question_label}), X NOT IN {prev_question_label})"
                
                if piping_filter:
                    piping_config["piping"] = piping_filter
            
            # Add piping_config to base_structure if it's not empty
            if piping_config:
                base_structure["optionConfig"] = piping_config



        if question_type == 'hidden_variable':
            definitions = []
            hidden_var_data = question_data.get('hidden_variable', {})
            
            # Process hidden_variable dictionary to create definitions
            # Instead of using enumerate, we'll use the value_label directly
            for value_label, condition in hidden_var_data.items():
                definitions.append({
                    "value": value_label,  # Use the year range as the value instead of sequential numbers
                    "condition": condition
                })
            
            return {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "variable",
                "issue": None,
                "label": label,
                "config": {},
                "mdText": convert_lbr_to_newline(question_text),
                "optout": {
                    "text": None,
                    "allowed": False
                },
                "created": current_time,
                "dynamic": False,
                "options": [],
                "updated": current_time,
                "condition": None,
                "confidence": 0.9,
                "definitions": definitions,
                **base_structure
            }


        if question_type in ['van_westendorp', 'van_westerndorp']:
            # Get the label from question data
            label = format_label(question_data.get('questionLabel', '') or question_data.get('qstnLabel', ''))
            
            # Initialize config with inputs array and currency from input data
            van_westendorp_config = {
                "inputs": [],
                "currency": question_data.get('currency', 'USD')  # Get currency from input, default to USD
            }
            
            # Process van_westendorp questions into inputs
            for i, question_text in enumerate(question_data.get('van_westerndorp', []), start=1):
                van_westendorp_config["inputs"].append({
                    "id": generate_nano_id(),
                    # "label": f"{i:02d}",
                    "label": str(i),
                    "helper": question_text
                })

            question_text_for_van_westerndorp=question_data.get("question","At what price do you think the product is too expensive?")
            
            # Get ask_logic condition if it exists
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            return {
                "id": key_id or generate_nano_id(),
                "text": question_text_for_van_westerndorp,
                "type": "van-westendorp",
                "label": label,
                "config": van_westendorp_config,
                "mdText": convert_lbr_to_newline(question_text_for_van_westerndorp),
                "created": current_time,
                "subtype": "",
                "updated": current_time,
                "condition": condition,
                **base_structure
            }



        if question_type == 'multi_percentage':
            # Get the label from question data
            
            label = format_label(question_data.get('questionLabel', '') or question_data.get('qstnLabel', ''))
            
            # Initialize config with inputs array
            multi_percentage_config = {
                "inputs": []
            }
            
            # Process options into inputs
            for i, option_dict in enumerate(question_data.get('options', []), start=1):
                for option_text, _ in option_dict.items():
                    multi_percentage_config["inputs"].append({
                        "id": generate_nano_id(),
                        # "label": f"{i:02d}",
                        "label":str(i),
                        "helper": option_text
                    })
            
            # Get ask_logic condition if it exists
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            return {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "multi-percentage",
                "label": label,
                "config": multi_percentage_config,
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "subtype": "",
                "updated": current_time,
                "condition": condition,
                **base_structure
            }

        if question_type == 'ranking':

            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            processed_options = []
            for i, option_dict in enumerate(options, start=1):
                for option_text, option_logic in option_dict.items():
                    # Determine option type
                    option_type = "user"
                    if "specify" in option_text.lower():
                        option_type = "other"
                        
                    processed_option = {
                        "id": generate_nano_id(),
                        "text": option_text,
                        "type": option_type,
                        "label": str(i),
                        "order": i - 1,
                        "mdText": convert_lbr_to_newline(option_text)
                    }
                    processed_options.append(processed_option)

            return {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "ranking",
                "label": label,
                "config": {
                    "randomize": question_data.get('randomize', False)
                },
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "linked_options": variable_list_keyword if variable_list_keyword else None,
                "options": processed_options,
                "subtype": "",
                "updated": current_time,
                "condition":condition,
                **base_structure
            }
        
        if question_type == 'single_text':
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            processed_question = {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "text",
                "label": label,
                "config": {
                    "ui": None,
                    "randomize": False
                },
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "updated": current_time,
                "skips": [],
                "optout": {"text": None, "allowed": False},
                "dynamic": False,
                "options": [],
                "condition": condition,
                "subtype": "single",
                "confidence": 0.9,
                **base_structure
            }

            
            
            # Process text logic if present
            if 'single_text_logic' in question_data:
                process_text_logic(question_data, processed_question, label)
                
            return processed_question
        
        if question_type == 'zip_code' or question_type =='zipcode':
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            processed_question= {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "zipcode",
                "label": label,
                "config": {
                    "country": question_data.get('country', None)
                },
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "options": [],
                "updated": current_time,
                "httpStatus": 200,
                "condition":condition,
                **base_structure
            }

            if 'zip_code_logic' in question_data:
                process_zip_code_logic(question_data, processed_question, label)
                
            return processed_question
        

      

        if question_type == 'multi_text' or question_type=="multi-text":
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic

            options = question_data.get('options', [])
            config_inputs = []
            
            for index, option_dict in enumerate(options, 1):
                # Get the first (and only) key from each dictionary in the list
                option_name = next(iter(option_dict.keys())) if option_dict else ''
                # Check if option_name starts with NA
                helper_text = "" if option_name.startswith("NA") else option_name
                
                input_config = {
                    "id": generate_nano_id(),
                    "label": str(index),
                    "helper": helper_text
                }
                config_inputs.append(input_config)
            
            processed_question = {
                    "id": key_id or generate_nano_id(),
                    "text": question_text,
                    "type": "text",
                    "label": label,
                    "config": {
                        "inputs": config_inputs
                    },
                    "mdText": convert_lbr_to_newline(question_text),
                    "created": current_time,
                    "updated": current_time,
                    "skips": [],
                    "optout": {"text": None, "allowed": False},
                    "dynamic": False,
                    "options": [],
                    "condition": condition,
                    "subtype": "multi",
                    "confidence": 0.9,
                    **base_structure
                }
            
            
            # Process text logic if present
            if 'multi_text_logic' in question_data:
                process_text_logic(question_data, processed_question, label)
                
            return processed_question
        
        if question_type == 'ai-chat':
            # Get ask_logic condition if it exists
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            # Build the chat configuration structure directly from the input fields
            ai_chat_config = {
                "chat": {
                    "objective": question_data.get("objective",question_text),  # The objective comes from the question field
                    "openingMessage": question_data.get('opening_message', ''),
                    "closingMessage": question_data.get('closing_message', ''),
                    "questionLimit": question_data.get('questions_limit', 6),
                    "timeLimit": "",  # Default empty string as per example
                }
            }
            
            return {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "ai-chat",
                "label": label,
                "config": ai_chat_config,
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "subtype": "",
                "updated": current_time,
                "condition": condition,
                **base_structure
            }
            

    


        if question_type == 'contact_form':
            # Get the label from question data first
            label = format_label(question_data.get('questionLabel', '') or question_data.get('qstnLabel', ''))
            
            contact_config = {
                "inputs": []
            }
            
            default_fields = [
                ("firstname", "First Name"),
                ("lastname", "Last Name"),
                ("phone", "Phone Number"),
                ("email", "Email")
            ]
            
            # Use default fields if contact_form is empty
            if not question_data.get('contact_form'):
                for label_field, helper in default_fields:
                    contact_config["inputs"].append({
                        "id": generate_nano_id(),
                        "label": label_field,  # Changed from label to label_field to avoid confusion
                        "helper": helper
                    })
            else:
                # Keep the same field labels regardless of user input text
                for field in question_data['contact_form']:
                    if 'first' in field.lower():
                        contact_config["inputs"].append({
                            "id": generate_nano_id(),
                            "label": "firstname",
                            "helper": field
                        })
                    elif 'last' in field.lower():
                        contact_config["inputs"].append({
                            "id": generate_nano_id(),
                            "label": "lastname",
                            "helper": field
                        })
                    elif 'email' in field.lower():
                        contact_config["inputs"].append({
                            "id": generate_nano_id(),
                            "label": "email",
                            "helper": field
                        })
                    elif 'phone' in field.lower():
                        contact_config["inputs"].append({
                            "id": generate_nano_id(),
                            "label": "phone",
                            "helper": field
                        })
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic

            return {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "contact-form",
                "label": label,
                "config": contact_config,
                "mdText": convert_lbr_to_newline(question_text),
                "optout": {
                    "text": "I do not want to provide my information",
                    "allowed": True
                },
                "allowed": True,
                "created": current_time,
                "subtype": "",
                "updated": current_time,
                "condition":condition,
                **base_structure
            }
        

        if question_type == 'currency':
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            processed_question = {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "currency",
                "label": label,
                "config": {
                    "currency": "USD",  # Default currency
                    "ui": "input",
                    "randomize": False
                },
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "updated": current_time,
                "skips": [],
                "optout": {"text": None, "allowed": False},
                "dynamic": False,
                "options": [],
                "condition": condition,
                "subtype": "",
                "confidence": 0.9,
                **base_structure  # Include any base structure elements
            }

            # Set currency if specified (only allow USD or CNY)
            if 'currency' in question_data:
                specified_currency = question_data['currency']
                processed_question["config"]["currency"] = specified_currency if specified_currency in ["USD", "CNY"] else "USD"

            # Handle data range if present
            if 'data_range' in question_data:
                processed_question["config"]["datarange"] = question_data['data_range']

            # Process currency logic if present
            if 'currency_logic' in question_data:
                process_currency_logic(question_data, processed_question, label)

            return processed_question
            



        


        if question_type == 'max_diff':
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            result = process_max_diff_question(question_data, label, current_time,raw_data)
            result["linked_options"] = variable_list_keyword if variable_list_keyword else None
            result["condition"]=condition
            if extra_text:
                result.update(base_structure)
            return result

        if question_type == 'unknown':
            return {
                "id": key_id or generate_nano_id(),
                "created": int(uuid.uuid1().time * 1e-3),
                "type": "unknown",
                "text": question_text,
                "mdText": convert_lbr_to_newline(question_text),
                "label": label,
                "options": [],
                "updated": current_time,
                "subtype": None,
                **base_structure
            }
        

        if question_type == 'multi-select-grid' or question_type=="multi_select_grid":
            print("asljndaskjdnaksj was called")
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            processed_question = {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "grid",
                "label": label,
                "config": {
                    "ui": None,
                    "randomize": question_data.get('randomize', False)
                },
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "updated": current_time,
                "skips": [],
                "optout": {"text": None, "allowed": False},
                "dynamic": False,
                "linked_options": variable_list_keyword if variable_list_keyword else None,
                "options": {"vertical": {"options": []}, "horizontal": {"options": []}},
                "condition": condition,
                "subtype": "multi",
                "confidence": 0.9,
                **base_structure
            }
            
            grid_options, skips = process_grid_question(question_data, label)
            processed_question["options"] = grid_options
            processed_question["skips"].extend(skips)
            return processed_question
        

        if question_type == 'grid':
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            processed_question = {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "grid",
                "label": label,
                "config": {
                    "ui": None,
                    "randomize": question_data.get('randomize', False)
                },
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "updated": current_time,
                "skips": [],
                "optout": {"text": None, "allowed": False},
                "dynamic": False,
                "linked_options": variable_list_keyword if variable_list_keyword else None,
                "options": {"vertical": {"options": []}, "horizontal": {"options": []}},
                "condition": condition,
                "subtype": "",
                "confidence": 0.9,
                **base_structure
            }
            
            grid_options, skips = process_grid_question(question_data, label)
            processed_question["options"] = grid_options
            processed_question["skips"].extend(skips)
            return processed_question

        if question_type == 'percentage_sum':
            processed_inputs = []
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            for i, option_dict in enumerate(options, start=1):
                for option_text, option_data in option_dict.items():
                    # We no longer need to process input_format since it's not used in the new structure
                    processed_input = {
                        "id": generate_nano_id(),
                        "label": str(i),  # Changed from f"{i:02d}" to match the desired format
                        "helper": option_text  # The option text becomes the helper text
                    }
                    processed_inputs.append(processed_input)
            
            return {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "percent-sum",
                "label": label,
                "config": {
                    "inputs": processed_inputs  # Put the processed inputs here instead of at root level
                },
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "updated": current_time,
                "condition":condition,
                "subtype": ""  ,
                **base_structure
            }

        if question_data['type'] == 'nps':
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            processed_question = {
                "id": key_id or generate_nano_id(),
                "created": int(uuid.uuid1().time * 1e-3),
                "type": "nps",
                "text": question_text,
                "mdText": convert_lbr_to_newline(question_text),
                "label": label,
                "options": [],
                "updated": current_time,
                "subtype": None,
                "condition": condition,
                "skips": [],  # Add this for skip logic
                **base_structure
            }
            
            # Process NPS logic if present
            if 'nps_logic' in question_data:
                process_nps_logic(question_data, processed_question, label)
            
            return processed_question
        
        if question_data['type'] == 'message' or question_data["type"]=="transition":
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            return {
                "id": key_id or generate_nano_id(),
                "created": int(uuid.uuid1().time * 1e-3),
                "type": "transition",
                "text": question_text,
                "mdText": convert_lbr_to_newline(question_text),
                "label": label,
                "options": [],
                "updated": current_time,
                "subtype": None,
                "condition":condition,
                **base_structure
            }
        
        if question_data['type'] == 'terminate':
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            return {
                "id": key_id or generate_nano_id(),
                "created": int(uuid.uuid1().time * 1e-3),
                "type": "terminate",
                "text": question_text,
                "mdText": convert_lbr_to_newline(question_text),
                "label": label,
                "options": [],
                "updated": current_time,
                "subtype": None,
                "condition":condition,
                **base_structure
            }
        
        # Add this inside process_single_question function, with other if conditions
        if question_type == 'single_select' or question_type=="single-select":
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            processed_question = {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "select",
                "label": label,
                "config": {
                    "ui": "dropdown" if question_data.get('dropdown', False) else "radio",
                    "randomize": question_data.get('randomize', False)
                },
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "updated": current_time,
                "skips": [],
                "optout": {"text": None, "allowed": False},
                "dynamic": False,
                "linked_options": variable_list_keyword if variable_list_keyword else None,
                "options": [],
                "condition": condition,
                "subtype": "single",
                "confidence": 0.9,
                **base_structure
            }

            if 'randomize_group' in question_data and isinstance(question_data['randomize_group'], list) and question_data['randomize_group']:
                processed_question["config"]["options"] = {
                    "groups": [
                        {
                            "id": generate_nano_id(),
                            "labels": [str(i) for i in question_data['randomize_group']],
                            "randomize": True
                        }
                    ]
                }

            # Process options if present
            if 'options' in question_data:
                for i, option_dict in enumerate(question_data['options'], start=1):
                    for option_text, option_logic in option_dict.items():
                        # option_label = f"{i:02d}"
                        option_label = str(i)
                        
                        # Extract backtick content and clean text
                        cleaned_text, backtick_value = extract_logic_from_backticks(option_text)

                        if use_ami:
                            cleaned_text=process_text(openai_model,cleaned_text,question_type=question_type)
                        if use_ami:
                            if option_logic == "other":
                                if "please specify" not in option_text.lower():
                                    option_logic = None
                        # Determine option type
                        option_type = "user"

                        option_type = "user"  # default type
                        if option_logic == "dontknow":
                            option_type = "dontknow"
                        elif option_logic == "notapplicable":
                            option_type = "notapplicable"
                        elif option_logic == "optout":
                            option_type = "optout"
                        elif option_logic == "nota":
                            option_type = "nota"
                        elif option_logic == "other":
                            option_type = "other"
                        elif "prefer not to answer" in cleaned_text.lower():
                            option_type = "optout"
                        elif "not applicable" in cleaned_text.lower():
                            option_type = "notapplicable"
                        
                        
                        
                        # Initialize skip dictionary if there's a backtick value
                        skip_dict = {backtick_value: None} if backtick_value else None
                        
                        # Create option
                        processed_option = {
                            "id": generate_nano_id(),
                            "text": cleaned_text,
                            "type": option_type,
                            "label": option_label,
                            "order": i - 1,
                            "mdText": convert_lbr_to_newline(cleaned_text),
                            "skip": skip_dict
                        }
                        
                        processed_question["options"].append(processed_option)
                        
                        # Process skip logic if present
                        if option_logic and option_logic != 'None':
                            if option_logic == 'THANK AND END':
                                skip = create_skip(f"{label} == `{option_label}`", "DISQUALIFY")
                                processed_question["skips"].append(skip)
                                # Update the skip ID in the option
                                if backtick_value:
                                    processed_option["skip"][backtick_value] = skip["id"]
                            elif option_logic.startswith('SKIP TO '):
                                skip_to = option_logic.replace('SKIP TO ', '').strip()
                                if label.isdigit() and skip_to.startswith('Q') and skip_to[1:].isdigit():
                                    skip_to = skip_to[1:]
                                skip = create_skip(f"{label} == `{option_label}`", "skip", skip_to)
                                processed_question["skips"].append(skip)
                                # Update the skip ID in the option
                                if backtick_value:
                                    processed_option["skip"][backtick_value] = skip["id"]

            return processed_question
        

        if question_type == 'year':
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            processed_question = {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "year",
                "label": label,
                "config": {
                    "ui": None,
                    "randomize": False
                },
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "updated": current_time,
                "skips": [],
                "optout": {"text": None, "allowed": False},
                "dynamic": False,
                "options": [],
                "condition": condition,
                "subtype": "",
                "confidence": 0.9,
                **base_structure
            }

            # Handle data range if present
            if 'data_range' in question_data:
                processed_question["config"]["datarange"] = question_data['data_range']

            # Process year logic if present
            if 'year_logic' in question_data:
                process_year_logic(question_data, processed_question, label)

            return processed_question
        

        if question_type == 'percentage':
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            processed_question = {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "percentage",
                "label": label,
                "config": {
                    "ui": None,
                    "randomize": False
                },
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "updated": current_time,
                "skips": [],
                "optout": {"text": None, "allowed": False},
                "dynamic": False,
                "options": [],
                "condition": condition,
                "subtype": "",
                "confidence": 0.9,
                **base_structure
            }

            # Handle data range if present
            if 'data_range' in question_data:
                processed_question["config"]["datarange"] = question_data['data_range']

            # Process percentage logic if present
            if 'percentage_logic' in question_data:
                process_percentage_logic(question_data, processed_question, label)

            return processed_question
        

        if question_type == 'multi_select' or question_type=="multi-select":
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            processed_question = {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "select",
                "label": label,
                "config": {
                    "ui": "dropdown" if question_data.get('dropdown', False) else "checkbox",
                    "randomize": question_data.get('randomize', False)
                },
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "updated": current_time,
                "skips": [],
                "optout": {"text": None, "allowed": False},
                "dynamic": False,
                "linked_options": variable_list_keyword if variable_list_keyword else None,
                "options": [],
                "condition": condition,
                "subtype": "multi",
                "confidence": 0.9,
                **base_structure
            }

            if 'randomize_group' in question_data and isinstance(question_data['randomize_group'], list) and question_data['randomize_group']:
                processed_question["config"]["options"] = {
                    "groups": [
                        {
                            "id": generate_nano_id(),
                            "labels": [str(i) for i in question_data['randomize_group']],
                            "randomize": True
                        }
                    ]
                }

            # Process options if present
            if 'options' in question_data:
                for i, option_dict in enumerate(question_data['options'], start=1):
                    for option_text, option_logic in option_dict.items():
                        # option_label = f"{i:02d}"
                        option_label = str(i)

                        
                        # Extract backtick content and clean text
                        cleaned_text, backtick_value = extract_logic_from_backticks(option_text)

                        if use_ami:
                            cleaned_text=process_text(openai_model,cleaned_text,question_type=question_type)
                        
                        # Determine option type
                        if use_ami:
                            if option_logic == "other":
                                if "please specify" not in option_text.lower():
                                    option_logic = None
                        # Determine option type
                        option_type = "user"
                        
                        
                        option_type = "user"  # default type
                        if option_logic == "dontknow":
                            option_type = "dontknow"
                        elif option_logic == "notapplicable":
                            option_type = "notapplicable"
                        elif option_logic == "optout":
                            option_type = "optout"
                        elif option_logic == "nota":
                            option_type = "nota"
                        elif option_logic == "other":
                            option_type = "other"
                        elif "prefer not to answer" in cleaned_text.lower():
                            option_type = "optout"
                        elif "not applicable" in cleaned_text.lower():
                            option_type = "notapplicable"
                        elif "other" in cleaned_text.lower():
                            option_type = "other"
                        
                        # Initialize skip dictionary if there's a backtick value
                        skip_dict = {backtick_value: None} if backtick_value else None
                        
                        # Create option
                        processed_option = {
                            "id": generate_nano_id(),
                            "text": cleaned_text,
                            "type": option_type,
                            "label": option_label,
                            "order": i - 1,
                            "mdText": convert_lbr_to_newline(cleaned_text),
                            "skip": skip_dict
                        }
                        
                        processed_question["options"].append(processed_option)
                        
                        # Process skip logic if present
                        if option_logic and option_logic != 'None':
                            if option_logic == 'THANK AND END':
                                skip = create_skip(f"{label} == `{option_label}`", "DISQUALIFY")
                                processed_question["skips"].append(skip)
                                # Update the skip ID in the option
                                if backtick_value:
                                    processed_option["skip"][backtick_value] = skip["id"]
                            elif option_logic.startswith('SKIP TO '):
                                skip_to = option_logic.replace('SKIP TO ', '').strip()
                                if label.isdigit() and skip_to.startswith('Q') and skip_to[1:].isdigit():
                                    skip_to = skip_to[1:]
                                skip = create_skip(f"{label} == `{option_label}`", "skip", skip_to)
                                processed_question["skips"].append(skip)
                                # Update the skip ID in the option
                                if backtick_value:
                                    processed_option["skip"][backtick_value] = skip["id"]

            return processed_question

        if question_data['type'] == 'notes':
            return {
                "id": generate_nano_id(),
                "created": int(uuid.uuid1().time * 1e-3),
                "type": "notes",
                "text": question_text,
                "mdText": convert_lbr_to_newline(question_text),
                "label": label,
                "options": [],
                "updated": current_time,
                "subtype": None,
                **base_structure
            }

        mapped_type = map_type(question_type, has_options)

        processed_question = {
            "id": key_id or generate_nano_id(),
            "text": question_text,
            "type": mapped_type,
            "label": label,
            "config": {
                "ui": "dropdown" if question_data.get('dropdown', False) else map_config_ui(question_type),
                "randomize": question_data.get('randomize', False)
            },
            "mdText": convert_lbr_to_newline(question_text),
            "created": current_time,
            "updated": current_time,
            "skips": [],
            "optout": {"text": None, "allowed": False},
            "dynamic": False,
            "options": [],
            "condition": condition,
            "subtype": map_subtype(question_type),
            "confidence": 0.9,
            **base_structure
        }

        if question_type == 'number':
            ask_logic = question_data.get('ask_logic', '')
            condition = ask_logic.get('1', '') if isinstance(ask_logic, dict) else ask_logic
            
            processed_question = {
                "id": key_id or generate_nano_id(),
                "text": question_text,
                "type": "number",
                "label": label,
                "config": {
                    "ui": None,
                    "randomize": False
                },
                "mdText": convert_lbr_to_newline(question_text),
                "created": current_time,
                "updated": current_time,
                "skips": [],
                "optout": {"text": None, "allowed": False},
                "dynamic": False,
                "options": [],
                "condition": condition,
                "subtype": "",
                "confidence": 0.9,
                **base_structure
            }

            # Handle data range if present
            if 'data_range' in question_data:
                processed_question["config"]["datarange"] = question_data['data_range']

            # Process number logic if present 
            if 'number_logic' in question_data:
                for action, condition in question_data['number_logic'].items():
                    if action == 'Terminating':
                        skip = create_skip(f"{condition}", "DISQUALIFY")
                        processed_question["skips"].append(skip)
                    elif action.startswith('SKIP TO '):
                        skip_to = action.replace('SKIP TO ', '').strip()
                        if label.isdigit() and skip_to.startswith('Q') and skip_to[1:].isdigit():
                            skip_to = skip_to[1:]
                        skip = create_skip(f"{condition}", "skip", skip_to)
                        processed_question["skips"].append(skip)

            return processed_question

        
        return processed_question

    except Exception as e:
        print(f"Warning: Error processing question: {str(e)}")
        return {
            "id": key_id or generate_nano_id(),
            "created": int(uuid.uuid1().time * 1e-3),
            "type": "unknown",
            "text": question_data.get('question', ''),
            "mdText": convert_lbr_to_newline(question_data.get('question', '')),
            "label": label,
            "options": [],
            "updated": current_time,
            "subtype": None,
            **base_structure
        }

def process_option_texts(processed_question):
    updated_question = processed_question.copy()

    if 'options' not in updated_question:
        
        return updated_question

    if updated_question.get('type') == 'grid':
        return updated_question
        
    
    for option in updated_question['options']:
        if option.get('skip'):
            for key, value in option['skip'].items():
                if value is None:
                    option['text'] = f"{option['text']} {key}"
                    option['mdText'] = convert_lbr_to_newline(option['text'])
    
    return updated_question


import json
def clean_json_with_backticks(openai_model, prompt):
    parsed_text = openai_model.client_open_ai.chat.completions.create(
        model="gpt-4",
        temperature=0.0000000001,
        response_format={ "type": "json_object" },
        messages=[
            {"role": "system", "content": "You are a JSON cleaning expert."},
            {"role": "user", "content": f"""
            You will receive a JSON string that may have formatting issues. Your task is to:
            1. Clean and format this JSON string to make it valid
            2. Return the cleaned JSON object with the same structure but properly formatted
            
            Examples:
            Input: {{"Skip to Q8 IF Q9=="Anshu"": True}}
            Output: {{"Skip to Q8 IF Q9=='Anshu'": true}}
            
            Input: {{"Disqualify IF Q9=="Anshuman"": True}}
            Output: {{"Disqualify IF Q9=='Anshuman'": true}}
            
            Important:
            - Keep the same key structure but ensure valid JSON formatting
            - Convert Python-style True/False to JSON true/false
            - Fix quote issues (convert double quotes in values to single quotes if needed)
            - The output must be a valid JSON object
            
            Here is the JSON string to clean: {prompt}
            """}
        ]
    )
    content = parsed_text.choices[0].message.content
    try:
        result = json.loads(content)
        return result
    except json.JSONDecodeError as e:
        print(f"Error parsing cleaned JSON: {e}")
        return None


import json
from utils import retry_on_json_error


@retry_on_json_error
def ranking_question_AMI(oepnai_model, prompt,sync_layer, general_tagged):
        schema = """
        {
            "ask_logic": "", # Capture any "ask_if" logic that specifies when a question should be asked.
            "questionLabel": "string",  # Capture the label exactly as it appears.
            "question": "string",   # Capture the main question text that we will be showing to the user taking the survey.
            "options": [  # List of options (if any) with their associated logic if any
                {"full_option_text": "None"}
            ],
            "type":"ranking"
            "keymap": "string"
        }
        """

        parsed_text = oepnai_model.client_open_ai.chat.completions.create(
            model="gpt-4o",
            temperature=0.0000000001,
            response_format={ "type": "json_object" },
            messages=[
                {"role": "system", "content": "You are processing survey questions and will receive raw question text. Your task is to structure the question into a JSON format`"},
                {"role": "user", "content": f"""
                You are a detailed survey question parser. 

                The schema to be used is: {schema}
                Sync_json guides you on capturing question_text (which maps to question in our JSON), logics, and options. Use the provided output as a reference to ensure proper mapping. Your output must align with it, especially for question, options, and logics. sync_json is : {sync_layer}.
                
                Parsing Rules:
                
                1. Question Label:  
                - Extract the exact identifier at the start (e.g., "Q1.", "5Kn.", etc.).  
                - Keep it exactly as written.  

                2. Question Text:  
                - Capture the entire  question text. (even if it contains (Donot Ask) everything in the question text should be captured

                3. Ask If Logic:  
                - Capture any "ask if" condition that determines when a question should be asked.  
                - Store this in the `ask_logic` field.  

                4. Options:
                - Be smart enough to understand the options from the text.
                - Create a list of dictionaries where each option is a key and value as None
                - example {{"Option text": None}}
                - example {{"Option text (Skip to Q4)": None}}
                - example: {{"Option text Terminate": None}}
                Important: If options have numbers in front of them, use the order the numbers indicate:

                For 04. a 03. b 02. c 01. d, capture the order as (d, c, b, a) (descending).
                For 01. a 02. b 03. c 04. d, capture the order as (a, b, c, d) (ascending).

                Make sure to follow the order as mentioned in the input text if it has the numbering in front of it.
                
                

                5. Piping Logic

                If a question explicitly references options from a previous question, add a "piping" field. any question, which asks to fill it's options from previous question it means it is a a piping question. then please add a field piping with a dictionary where the  key is the questionLabel from which we have to fetch responses. and value could be any from these 4: [show selected, show not selected, hide selected, hide not selected].
                for example . you see something like this in a text: "Show all selected in QM. " then your json will have a field like : piping : {{"QM":"show selected" }}
                

                6. Port in response 
                
                If a question text references a previous question and requests to "pipe in" or "port in" results, follow these steps:
                Identify the referenced question.
                Example: If the prompt states:
                "Q3. Why do you like (port in response from Q2)?"
                Replace the placeholder with {{Q2}}, so the question becomes:
                "Why do you like {{Q2}}?"
                Ensure the label (e.g., Q2) matches the format of the referenced question.
                If the reference uses terms like "Recall #2" or similar, replace it with the corresponding question label in the format {{Q2}} (not {{Recall #2}}).
                Use the exact question label (e.g., Q2) as defined in the context.
                Do not include extra text or reference terms; only use the question label.

                7. Please capture the text after 'Ref' if it is given in the input text into a field called 'Ref'. For example, if the input text contains 'Ref:PHONEINTRO', then your output should include 'Ref': 'PHONEINTRO'.
            
                

                Examples:

                Input:
                ```Ask if Qn==1 \n
                QM. Demo Question (Disqualify if <1) Randomize \n random text
                1. option a 
                2. option b 
                3. option c 
                
                ```

                Output:
                {{
                    "ask_logic":"Ask if Qn==1",
                    "questionLabel": "QM",
                    "question": "Demo Question?",
                    "options": [
                        {{"option a ": None}},
                        {{"option b ": None}},
                        {{"option c ": None}}
                    ],
                    "type":"ranking",
                }}
                
        
                
             
                Important: please give me a valid json.
                Also, here are some special mentions by the user: pay special attention and include them as well: {general_tagged}.
                Parse the following survey question according to these rules: {prompt}
                """
                }
            ]
        )

        content = parsed_text.choices[0].message.content
        result = json.loads(content)
        return result


@retry_on_json_error
def ranking_logic_AMI(openai_model, input_json,user_logic, logic_tagged):
    

    parsed_text = openai_model.client_open_ai.chat.completions.create(
        model="gpt-4o",
        temperature=0.0000000001,
        response_format={ "type": "json_object" },
        messages=[
            {"role": "system", "content": "You are a precise survey question parser that captures every detail without losing any information."},
            {"role": "user", "content": f"""
            
            You will be receving a json which has ranking question , your task is to understand any logics if they exist and put it in the bucket of logics we currently support. 
            We currently support :
            1. Termination logics (it can be written in any way) , you should be smart enough to understand if that word/sentence means termination
            2. Skip logics (it can be written in any way) , you should be smart enough to understand if that word/sentence means skip.
            3. User defined logics : The user might have different way of writing any logics and we've captured that.
             
            You are not supposed to make any changes in the options.
                
            Important : The users way of writing the logic is : {user_logic} make sure to only use the words as logics which are explicitly mentioned
            please pay special attention while assigning logics. 

            You will primarily see logics captured in `{{}}`. you goal is to understand if that logic falls in the above 3 categories of logics we support for the ranking. 
            If there's no logic that we support. (rememebr there can be mutliple ways of writing any logic, we have to be smart enough to understand the bigger picture.).But if we dont support that logic or there is no logic, just return the json as it is. without making any changes.

            But if you see that there's any logic that we support 1. Termination, 2. Skip 3. User defined logics.  then we should follow the below steps.


            In the question if you observe any logic that we support. Then please add another field saying multi_text_logic and the key value pair will be the following. 
            For terminating the key should be "Terminating" and the value should be conditon but the condition will only be " [logical operator] [value]" 
            For skip conditions the key would be "SKIP TO [questionLabel of the question]" value would be " [logical operator] [value] " 
            important: The logical operator that you use should be like QuestionLabel ==, >=, <=, != , or whatever the option logic says. every opertor should have questionLabel in front of it. give me the exact logic that you get. you dont have to change any logic. make it both for Terminating or SKIP questions. follow the convention of writing "==, >=, <=, != " even if the user has just one "=" we should eb using "=="
            Important: Provide the exact logic, but include the questionLabel (retrieved from the JSON) before every operator. Ensure the operators are in uppercase, like OR and AND. By questionLabel, I mean the actual value of the questionLabel field from the JSON, not the literal term questionLabel.
            We also turn the None to True if its the logic we support and we've added a field for it in our json, like we did in the below example. 
            

            example: you got this:

             {{     
                    "ask_logic":"Ask if Q3==0",
                    "questionLabel": "QM",
                    "question": "Demo Question? `{{"(Disqualify if QM=="XYZ")": None}}` \n random text",
                    "options": [
                        {{"option a ": None}},
                        {{"option b ": None}},
                        {{"option c ": None}}
                    ],
                    "extra_text":"",
                    "type":"ranking",
                    
                    
                 
            }}

            your output will look something like this:

            {{  
                    "ask_logic":"Ask if Q3==0",
                    "questionLabel": "QM",
                    "question": "Demo Question? `{{"(Disqualify if QM=="XYZ")": True}}` \n random text",
                    "type":"ranking",
                    "options": [
                                {{"option a ": None}},
                                {{"option b ": None}},
                                {{"option c ": None}}
                            ],
                    "extra_text":"",
                    "type":"ranking",
                    "ranking_logic": {{"Terminating":"QM=="XYZ"}}
            
            }}
            

            example of a logic we dont understand.
            {{
                    "ask_logic":"Ask if Q3==0",
                    "questionLabel": "QM",
                    "question": "Demo Question? `{{"Some logic I dont know": None}}` \n random text",
                    "type":"ranking",
                    "options": [
                        {{"option a ": None}},
                        {{"option b ": None}},
                        {{"option c ": None}}
                    ],
                    "extra_text":"",
                    "type":"ranking",
                 
            }}
            we will return it as it :
            {{
                    "ask_logic":"Ask if Q3==0"
                    "questionLabel": "QM",
                    "question": "Demo Question? `{{"Some logic I dont know": None}}` \n random text",
                    "type":"ranking",
                    "options": [
                        {{"option a ": None}},
                        {{"option b ": None}},
                        {{"option c ": None}}
                    ],
                    "extra_text":"",
                    "type":"ranking",
                 
            }}

            Please pay special attention to the instructions for logics and make sure you mark it correctly, double check it. 
            Most Important: No data should be deletd, not even a single dot or a bracket.
            please give me the correct json. 
            Make sure to properly tag all special instructions without missing any. These instructions are crucial and must always be categorized correctly. Here are the instructions: {logic_tagged}
            The json is :{input_json}      


                    
            
            """
            }
        ]
    )

    content = parsed_text.choices[0].message.content
    result = json.loads(content)
    return result




from Functions.helper import create_skip, clean_option_text
from Functions.nano_id import *
import re



def process_text_logic(question, processed_question, label):
    """
    Process text logic for questions, supporting both TERMINATING and SKIP TO conditions.
    Cleans conditions by removing dots from labels.
    
    Args:
        question: The original question dictionary
        processed_question: The processed question dictionary being built
        label: The question label
    """
    if not question.get('text_logic'):
        return

    # Remove any dots from the label for use in conditions
    clean_label = label.replace('.', '')

    for action, condition in question['text_logic'].items():
        # Clean the condition by replacing label with dot with clean label
        clean_condition = condition.replace(f"{label}.", clean_label)
        
        if action.lower() == 'terminating':
            skip = create_skip(clean_condition, "DISQUALIFY")
            processed_question["skips"].append(skip)
        elif action.startswith('SKIP TO '):
            skip_to = action.replace('SKIP TO ', '').strip()
            # Handle Q prefix in skip_to target
            if label.isdigit() and skip_to.startswith('Q') and skip_to[1:].isdigit():
                skip_to = skip_to[1:]
            skip = create_skip(clean_condition, "skip", skip_to)
            processed_question["skips"].append(skip)







def process_year_logic(question, processed_question, label):
    for action, condition in question['year_logic'].items():
        if action == 'Terminating':
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            # Check if condition already contains the label
            if not condition.startswith(label):
                condition = f"{label} {condition}"
            skip = create_skip(condition, "DISQUALIFY")
            processed_question["skips"].append(skip)
        elif action.startswith('SKIP TO '):
            skip_to = action.replace('SKIP TO ', '').strip()
            if label.isdigit() and skip_to.startswith('Q') and skip_to[1:].isdigit():
                skip_to = skip_to[1:]
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            skip = create_skip(f"{condition.strip()}", "skip", skip_to)
            processed_question["skips"].append(skip)




def process_percentage_logic(question, processed_question, label):
    for action, condition in question['percentage_logic'].items():
        if action == 'Terminating':
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            # Check if condition already contains the label
            if not condition.startswith(label):
                condition = f"{label} {condition}"
            skip = create_skip(condition, "DISQUALIFY")
            processed_question["skips"].append(skip)
        elif action.startswith('SKIP TO '):
            skip_to = action.replace('SKIP TO ', '').strip()
            if label.isdigit() and skip_to.startswith('Q') and skip_to[1:].isdigit():
                skip_to = skip_to[1:]
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            skip = create_skip(f"{condition.strip()}", "skip", skip_to)
            processed_question["skips"].append(skip)

def process_text_logic(question, processed_question, label):
    for action, condition in question['single_text_logic'].items():
        if action == 'Terminating':
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            # Check if condition already contains the label
            if not condition.startswith(label):
                condition = f"{label} {condition}"
            skip = create_skip(condition, "DISQUALIFY")
            processed_question["skips"].append(skip)
        elif action.startswith('SKIP TO '):
            skip_to = action.replace('SKIP TO ', '').strip()
            if label.isdigit() and skip_to.startswith('Q') and skip_to[1:].isdigit():
                skip_to = skip_to[1:]
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            skip = create_skip(f"{condition.strip()}", "skip", skip_to)
            processed_question["skips"].append(skip)

def process_zip_code_logic(question, processed_question, label):
    # Initialize skips if it doesn't exist
    if "skips" not in processed_question:
        processed_question["skips"] = []

    for action, condition in question['zip_code_logic'].items():
        if action == 'Terminating':
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            
            # Remove dots from both the label and condition
            clean_label = label.replace('.', '')
            condition = condition.replace('.', '')
            
            # Check if condition already contains the label
            if not condition.startswith(clean_label):
                condition = f"{clean_label} {condition}"
            
            skip = create_skip(condition, "DISQUALIFY")
            processed_question["skips"].append(skip)
            
        elif action.startswith('SKIP TO '):
            skip_to = action.replace('SKIP TO ', '').strip()
            if label.isdigit() and skip_to.startswith('Q') and skip_to[1:].isdigit():
                skip_to = skip_to[1:]
            
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            
            skip = create_skip(f"{condition.strip()}", "skip", skip_to)
            processed_question["skips"].append(skip)





def process_nps_logic(question, processed_question, label):
    for action, condition in question['nps_logic'].items():
        if action == 'Terminating':
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            # Check if condition already contains the label
            if not condition.startswith(label):
                condition = f"{label} {condition}"
            skip = create_skip(condition, "DISQUALIFY")
            processed_question["skips"].append(skip)
        elif action.startswith('SKIP TO '):
            skip_to = action.replace('SKIP TO ', '').strip()
            if label.isdigit() and skip_to.startswith('Q') and skip_to[1:].isdigit():
                skip_to = skip_to[1:]
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            skip = create_skip(f"{condition.strip()}", "skip", skip_to)
            processed_question["skips"].append(skip)





def process_grid_options(options, label_prefix):
    """
    Process grid options that can be either:
    - A dictionary with multiple keys
    - A list of dictionaries with single keys
    """
    processed_options = []
    
    # If options is a list, handle as list of dictionaries
    if isinstance(options, list):
        for i, option_dict in enumerate(options, start=1):
            # Get the first (and only) key from the dictionary
            text = next(iter(option_dict))
            # Remove the "[XX]" prefix if it exists
            cleaned_text = clean_option_text(text)
            # label = f"{i:02d}"
            label = str(i)
            option = {
                "id": generate_nano_id(),
                "text": cleaned_text,
                "label": label,
                "mdText": cleaned_text,
                "type": "user",  # Adding required fields
                "order": i - 1,
                "skip": None     # Adding skip field as None by default
            }
            processed_options.append(option)
    # If options is a dictionary, handle as before
    elif isinstance(options, dict):
        for i, text in enumerate(options.keys(), start=1):
            # Remove the "[XX]" prefix if it exists
            cleaned_text = clean_option_text(text)
            # label = f"{i:02d}"
            label = str(i)
            option = {
                "id": generate_nano_id(),
                "text": cleaned_text,
                "label": label,
                "mdText": cleaned_text,
                "type": "user",  # Adding required fields
                "order": i - 1,
                "skip": None     # Adding skip field as None by default
            }
            processed_options.append(option)
            
    return processed_options

def process_grid_question(question, label):
    grid_options = {"vertical": {"options": []}, "horizontal": {"options": []}}
    
    # Process columns (vertical)
    if 'columns' in question:
        grid_options["vertical"]["options"] = process_grid_options(question['rows'], 'V')
    
    # Process rows (horizontal)
    if 'rows' in question:
        grid_options["horizontal"]["options"] = process_grid_options(question['columns'], 'H')
    
    # Handle cases where one of rows/columns is empty
    if not grid_options["vertical"]["options"] and 'options' in question:
        grid_options["vertical"]["options"] = process_grid_options(question['options'], 'V')
    elif not grid_options["horizontal"]["options"] and 'options' in question:
        grid_options["horizontal"]["options"] = process_grid_options(question['options'], 'H')
    
    return grid_options, []





def process_number_logic(question, processed_question, label):
    for action, condition in question['number_logic'].items():
        if action == 'Terminating':
            # Don't add label since condition already has it
            skip = create_skip(condition, "DISQUALIFY")
            processed_question["skips"].append(skip)
        elif action.startswith('SKIP TO '):
            skip_to = action.replace('SKIP TO ', '').strip()
            if label.isdigit() and skip_to.startswith('Q') and skip_to[1:].isdigit():
                skip_to = skip_to[1:]
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            cleaned_condition = re.sub(r'[^\d\s<>=]+', '', condition)
            skip = create_skip(cleaned_condition.strip(), "skip", skip_to)
            processed_question["skips"].append(skip)





def process_currency_logic(question, processed_question, label):
    for action, condition in question['currency_logic'].items():
        if action == 'Terminating':
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            # Check if condition already contains the label
            if not condition.startswith(label):
                condition = f"{label} {condition}"
            skip = create_skip(condition, "DISQUALIFY")
            processed_question["skips"].append(skip)
        elif action.startswith('SKIP TO '):
            skip_to = action.replace('SKIP TO ', '').strip()
            if label.isdigit() and skip_to.startswith('Q') and skip_to[1:].isdigit():
                skip_to = skip_to[1:]
            if isinstance(condition, list):
                condition = ''.join(str(x) for x in condition)
            skip = create_skip(f"{condition.strip()}", "skip", skip_to)
            processed_question["skips"].append(skip)


def currency_symbol_to_code(symbol):
    currency_mapping = {
        '$': 'USD',  # US Dollar
        '€': 'EUR',  # Euro
        '£': 'GBP',  # British Pound Sterling
        '¥': 'JPY',  # Japanese Yen
        '₹': 'INR',  # Indian Rupee
        '₽': 'RUB',  # Russian Ruble
        '₣': 'CHF',  # Swiss Franc
        '₺': 'TRY',  # Turkish Lira
        '₩': 'KRW',  # South Korean Won
        '₴': 'UAH',  # Ukrainian Hryvnia
        'A$': 'AUD',  # Australian Dollar
        'C$': 'CAD',  # Canadian Dollar
        'HK$': 'HKD',  # Hong Kong Dollar
        '₱': 'PHP',  # Philippine Peso
        '฿': 'THB',  # Thai Baht
        'R$': 'BRL',  # Brazilian Real
        '₪': 'ILS',  # Israeli New Shekel
        '₦': 'NGN',  # Nigerian Naira
        '₡': 'CRC',  # Costa Rican Colón
        '₫': 'VND',  # Vietnamese Dong
        '៛': 'KHR',  # Cambodian Riel
        '₸': 'KZT',  # Kazakhstani Tenge
        '₿': 'BTC',  # Bitcoin
        'Ξ': 'ETH',  # Ethereum
        '₼': 'AZN',  # Azerbaijani Manat
        '₾': 'GEL',  # Georgian Lari
        '₻': 'MGA',  # Malagasy Ariary
        '₽': 'RUB',  # Russian Ruble (duplicate, keeping for compatibility)
        '₺': 'TRY',  # Turkish Lira (duplicate, keeping for compatibility)
    }
    return currency_mapping.get(symbol, 'USD')




from CreateSection.Functions.generate_uuid import *
import time

def process_hidden_variables(question: dict) -> dict:
    """
    Process hidden variables in a question and create a new variable type question if needed.
    """
    if not question.get('hidden_variable') or not question.get("hidden-variable"):
        return None

    # Create a new question for the hidden variable
    hidden_var_question = {
        "id": generate_nano_id(),
        "text": question.get("question","Hidden Variable question"),
        "type": "variable",
        "issue": None,
        "label": f"{question['questionLabel']}HV",
        "config": {},
        "mdText": "Hidden Variable Question",
        "optout": {
            "text": None,
            "allowed": False
        },
        "created": int(uuid.uuid1().time * 1e-3),
        "dynamic": False,
        "options": [],
        "updated": int(time.time() * 1000),
        "condition": None,
        "confidence": 0.9,
        "definitions": []
    }

    # Process hidden variable definitions
    if isinstance(question['hidden-variable'], dict):
        for value, condition in question['hidden-variable'].items():
            hidden_var_question['definitions'].append({
                "value": value,
                "condition": condition
            })

    return hidden_var_question



def extract_conversation_from_metadata(metadata) -> str:
    """
    Extract conversation markdown from Metadata object
    
    Args:
        metadata (Metadata): Pydantic model containing previous_conversation_json
        
    Returns:
        str: The extracted conversation with messages separated by new lines
    """
    # Check if previous_conversation_json exists
    if not metadata.previous_conversation:
        return ""

    # Extract markdown from each message
    messages = []
    for message in metadata.previous_conversation:
        if "markdown" in message:
            messages.append(message["markdown"])
    
    # Join messages with double newlines
    return "\n\n".join(messages)


def get_question_labels(payload):
    # Initialize an empty list to store the labels
    labels = []
    
    # Check if payload and blocks exist
    if not payload or 'blocks' not in payload or not isinstance(payload['blocks'], list):
        return labels
    
    # Iterate through each block
    for block in payload['blocks']:
        # Check if questions array exists in the block
        if 'questions' in block and isinstance(block['questions'], list):
            # Extract label from each question and add to the list
            for question in block['questions']:
                if 'label' in question:
                    labels.append(question['label'])
    
    return labels


# import json
# from utils import retry_on_json_error

# @retry_on_json_error
# def finalize_survey_objective(openai_model, initial_objective,conversation_history):
#     final_objective_template = f"""
#     {   "general_response":"string",
#         "final_objective":"string"
#     }
#     """
#     parsed_text = openai_model.client_open_ai.chat.completions.create(
#         model="gpt-4o",
#         temperature=0.0000000001,
#         response_format={ "type": "json_object" },
#         messages=[
#             {"role": "system", "content": "You are a specialized agent responsible for evaluating and refining survey objectives. Your task is to analyze the past interactions, extract key details, and generate a well-structured final objective for the survey."},
#             {"role": "user", "content": f"""
#             The user has provided the following initial objective for the survey: {initial_objective}
#             {f"Below is the conversation history between the user and the system regarding their survey objective, including the initial objective and some follow-up interactions: {conversation_history}" if conversation_history else ""}

#             Based on this, your task is to:
#             1. Evaluate the conversation history and extract all relevant details.
#             2. Eliminate redundancies and ensure clarity.
#             3. Synthesize a final, well-structured survey objective, which should include:
#                - A clear and concise objective statement
#                - The target audience
#                - The key themes or topics the survey will cover
#                - The expected outcome or insights the survey aims to achieve

#             Your response should be a single refined objective that incorporates all relevant details in a professional and structured manner, if you dont have some information, please make sure to make an objective based on the information you have. you are the expert in this field and you should make the best use of it.
#             give me valid json response.
#             In the general_response, please give the intial response the the user it is just a message stating the user that as per your understanding the user wants to achieve the following objective, please give the final objective.
#             please follow the following format for the response: {final_objective_template}
#             """}
#         ]
#     )

#     content = parsed_text.choices[0].message.content
#     result = json.loads(content)
#     return result



import json
from utils import retry_on_json_error

@retry_on_json_error
def finalize_survey_objective(openai_model, initial_objective, conversation_history):
    final_objective_template = """
{ "general_response":"string",
"final_objective":"string"
}
    """
    
    # Build the conversation history part
    conversation_part = ""
    if conversation_history:
        conversation_part = f"Below is the conversation history between the user and the system regarding their survey objective, including the initial objective and some follow-up interactions: {conversation_history}"
    
    parsed_text = openai_model.client_open_ai.chat.completions.create(
        model="gpt-4o",
        temperature=0.0000000001,
        response_format={ "type": "json_object" },
        messages=[
            {"role": "system", "content": "You are a specialized agent responsible for evaluating and refining survey objectives. Your task is to analyze the past interactions, extract key details, and generate a well-structured final objective for the survey."},
            {"role": "user", "content": f"""
The user has provided the following initial objective for the survey: {initial_objective}
{conversation_part}
Based on this, your task is to:
1. Evaluate the conversation history and extract all relevant details.
2. Eliminate redundancies and ensure clarity.
3. Synthesize a final, well-structured survey objective, which should include:
- A clear and concise objective statement
- The target audience
- The key themes or topics the survey will cover
- The expected outcome or insights the survey aims to achieve
Your response should be a single refined objective that incorporates all relevant details in a professional and structured manner, if you dont have some information, please make sure to make an objective based on the information you have. you are the expert in this field and you should make the best use of it.
give me valid json response.
In the general_response, please give the intial response the the user it is just a message stating the user that as per your understanding the user wants to achieve the following objective, please give the final objective.
please follow the following format for the response: {final_objective_template}
            """}
        ]
    )
    content = parsed_text.choices[0].message.content
    result = json.loads(content)
    return result


# for hidden_variable logic: please identify the logic (it is majoryly as a grouping logic), give me a dictionary that has key as the name of the bucket like "18-34"  (keep the name of the bucket as it is.) and value as the logic "Q5 IN [`01`,`02`] where 01 and 02 are the options labels and Q5 would be the qquestion label of that question.(assign option numbers in a serial way 01, 02 to the options accordingly.) if not the option type logic then it can be something like "Q5 > 30" where Q5 is the question Label. 
#                 # be sure to smartly understand what the user is trying to mean with that option. There can be multiple ways to say all these logics.


import json

def hidden_variable_question(oepnai_model, prompt):
        schema = """
        {
            "ask_logic": "", # Capture any "ask_if" logic that specifies when a question should be asked.
            "questionLabel": "string",  # Capture the label exactly as it appears, even is there's any text till we start the options, should be captured as it is. we dont want to lose any text at all, not even a single dot or bracket.
            "question": "string",  # Full question text with any logic in `{ logic: None }` format . YOU ARE NOT SUPPOSED TO DELETE ANYTHING COPY IT AS IT IS. 
            "type"="hidden_variable",
            "extra_text": "string", # Any text that does not fit into the specified categories should be captured here. The purpose of "extra_text" is to ensure no data is lost, allowing us to retain every detail from the input text, including any text or symbols that fall outside the defined structure. (if there's any logic, then make sure to wrap it in `{logic: None}`)

        }
        """

        parsed_text = oepnai_model.client_open_ai.chat.completions.create(
            model="gpt-4o",
            temperature=0.0000000001,
            response_format={ "type": "json_object" },
            messages=[
                {"role": "system", "content": "You are a precise survey question parser that captures every detail without losing any information. The most important thing is that we should not delete even a single dot. You are dealing with nps question type"},
                {"role": "user", "content": f"""
                You are a detailed survey question parser. Your task is to analyze survey questions and capture EVERY piece of text while structuring them into a specific JSON format. Follow these guidelines:

                CORE PRINCIPLES:
                1. Never delete or modify any text. WE SHOULD NOT LOSE EVEN A SINGLE DOT. we cannot delete any text at any cost.
                2. Preserve the exact structure and formatting
                3. DONOT EVEN DELETE A SINGLE BRACKET.
                 
                The schema is : {schema}

                PARSING RULES:

                1. Question Label:
                - Extract the exact identifier at the start (e.g., "Q1.", "5Kn.", etc.)
                - Keep it exactly as written

                2. Question Text:
                - Include everything from the start of the question to just before the options start (if they exist). anything and everything, even if it doent make sense will be a part of the question text.
                - Even random or unrelated text must be part of the question.
                - For any logic/instructions in parentheses or brackets, wrap them in `{{"text": None}}` format
                - Example: "On a scale of 0 to 10, how likely are you to recommend our product/service to a friend or colleague? (Skip to Q3 if ==3)" becomes:
                "On a scale of 0 to 10, how likely are you to recommend our product/service to a friend or colleague? `{{"(Skip to Q3 if ==3)":None}}"

                

                3. Text After Options:
                - Capture any text that appears after the options list
                - Wrap any logic in `{{"text": None}}` format
                - Important: Text after options will exist even if there are no options, its there to capture the ending text. so that we dont lose any text.


                Important: please identify the logic (it is majoryly as a grouping logic), give me a dictionary "hidden_variable" that has key as the name of the bucket like "18-34"  (keep the name of the bucket as it is.) and value as the logic "Q5 IN [`01`,`02`] where 01 and 02 are the options labels and Q5 would be the qquestion label of that question.(assign option numbers in a serial way 01, 02 to the options accordingly.) if not the option type logic then it can be something like "Q5 > 30" where Q5 is the question Label. 
                make sure the values in the hidden_variable follow the pattern of "Q5 IN [`01`,`02`]" where we basically tell them that it belong to the group if in Q5 we selected 01 or 02. using this: "Q5 IN [`01`,`02`]"
          
                
                SPECIAL INSTRUCTIONS:
                
                - All logic must be wrapped in `{{}}` and quoted properly
                - Keep all original spacing and formatting
                - Never remove any text, even if it seems redundant, we should be capturing each and every detail. we should not delete even a single dot.

                
                important: make sure to capture logics in the `{{}}`
                Important: the text of the options will have the complete text and the value of the options will always be None. in the options you dont need to capture any logic separately. just capture the entire text as it is without deleting anything. we should not delete anything.
                Important : the value in the options will always be None
                Important: a question will be until the options start. so if there's anything till the options start should be captured in the question.
 
                
                CORE PRINCIPLES and MOST IMPORTANT:

                Never delete or modify any text: Every part of the input must be accounted for. Missing even one word is a critical error.
                Preserve structure and format: The output must follow the specified JSON schema exactly.
                
                The most important thing is that we should not lose any text!!
                Most important: Dont get stuck, if by any chance you're confused give me whatever you understand. just dont delete anything and dont be stuck
                Finally, ensure that every piece of text from the input has been captured in the JSON without any loss. If any part of the text does not fit into the defined schema, it should be included in the extra_text field. The goal is to retain all input text in its entirety.
                Parse the following survey question according to these rules:  {prompt}
                """
                }
            ]
        )

        content = parsed_text.choices[0].message.content
        result = json.loads(content)
        return result



# def hidden_variable_logic(openai_model, input_json,user_logic):
    

#     parsed_text = openai_model.client_open_ai.chat.completions.create(
#         model="gpt-4o",
#         temperature=0.0000000001,
#         response_format={ "type": "json_object" },
#         messages=[
#             {"role": "system", "content": "You are a precise survey question parser that captures every detail without losing any information."},
#             {"role": "user", "content": f"""
            



#             You will be receving a json which has nps question , your task is to understand any logics if they exist and put it in the bucket of logics we currently support.
                
            
#             be sure to smartly understand what the user is trying to mean with that option. There can be multiple ways to say all these logics.
#             Important : Apart from that: Ill be sharing with you the users way of writing a logic for the survey. So you should be smart enough to understand what the user is trying to say and then make our json.
#             Important : The users way of writing the logic is : {user_logic} make sure to only use the words as logics which are explicitly mentioned
#             please pay special attention while assigning logics. 
#             Be sure to understand how a user writes the logic. 
#             Use you general understanding and the user_logic way of writing the logic. 
#             Also, if the logic is not in english. please understand it properly
        
#             Very Important: You are not supposed to add any new options on your own, just deal with whatever you have.
#             Most Important: No data should be deletd, not even a single dot.
#             Important : you are not supposed to change the logic field in the json.
            
#             In the question if you observe any logic associated with it. please add another field saying nps_logic and for the key,value pair. For terminating the key should be Terminating and the value should be conditon but the condition will only be " [logical operator] [value]" and for skip conditions the key would be [ SKIP TO [qstnLabel] value would be " [logical operator] [value] " . 
#             The logical operator that you use should be like QuestionLabel ==, >=, <=, != , or whatever the option logic says. every opertor should have questionLabel in front of it. give me the exact logic that you get. you dont have to change any logic. make it both for Terminating or SKIP questions. Important: give me the exact logic but with questionLabel (pick it up from json) in front of every operator. keep the operators in capital like "OR"."AND". also by questionLabel : I mean the questionLabel of the question. and not "questionLabel".
                
#             Please pay special attention to the instructions for logics and make sure you mark it correctly, double check it. 
#             Important: Donot even delete a single bracket.
#             Most Important: No data should be deletd, not even a single dot.
#             The json is :{input_json}      

#             please give me the correct json. 
#             """
#             }
#         ]
#     )

#     content = parsed_text.choices[0].message.content
#     result = json.loads(content)
#     return result




import copy
def delete_sections(data, section_name_to_delete: str):
    """
    Delete a section (block) from the survey data structure
    
    Args:
        data: The complete_survey_data (JsonPayload model or dict)
        section_name_to_delete: Name of the section to delete
        
    Returns:
        dict: The updated data in dictionary format, ready for JSON serialization
    """
    # Create a deep copy to avoid modifying the original data
    if hasattr(data, "dict") and callable(data.dict):
        # It's a Pydantic model
        data_dict = data.dict()
    else:
        # It's already a dict or something else
        data_dict = copy.deepcopy(data)
    
    # Check if blocks exist
    if "blocks" in data_dict:
        # Get the question IDs that will be deleted
        deleted_question_ids = []
        for block in data_dict["blocks"]:
            if block.get("name") == section_name_to_delete:
                questions = block.get("questions", [])
                deleted_question_ids = [q.get("id") for q in questions if q.get("id")]
                break
        
        # Filter out the block with the matching name
        original_blocks_length = len(data_dict["blocks"])
        data_dict["blocks"] = [
            block for block in data_dict["blocks"]
            if block.get("name") != section_name_to_delete
        ]
        new_blocks_length = len(data_dict["blocks"])
        
        # Also update annotated_data if any block was removed
        if (original_blocks_length > new_blocks_length and 
            "annotated_data" in data_dict and 
            deleted_question_ids):
            data_dict["annotated_data"] = [
                item for item in data_dict["annotated_data"]
                if item.get("id") not in deleted_question_ids
            ]
    
    # Always return a dictionary, not a Pydantic model
    return data_dict



def delete_question_by_label(json_payload, label_to_delete):
    """
    Deletes a question with the specified label from both the survey questions and annotated data.
    
    Args:
        json_payload: The JSON payload containing blocks and questions
        label_to_delete: The label of the question to delete (e.g., "Q5" or "Q5.")
        
    Returns:
        The modified JSON payload with the question removed
    """
    # Normalize the label by removing trailing periods
    normalized_label = label_to_delete.rstrip('.')
    question_id_to_delete = None
    
    # Check if we have blocks with questions
    if hasattr(json_payload, "blocks"):
        blocks = json_payload.blocks
    elif isinstance(json_payload, dict) and "blocks" in json_payload:
        blocks = json_payload["blocks"]
    else:
        return json_payload  # No blocks found, return unchanged
    
    # Step 1: Find and remove the question from blocks
    for block in blocks:
        if "questions" in block:
            # Find the question to delete and get its ID
            for index, question in enumerate(block["questions"]):
                if "label" in question and question["label"]:
                    question_label = question["label"].rstrip('.')
                    if question_label == normalized_label:
                        # Store the ID for annotated_data deletion
                        question_id_to_delete = question.get("id")
                        # Remove the question from the list
                        block["questions"].pop(index)
                        break
    
    # Step 2: Remove the corresponding entry from annotated_data
    if question_id_to_delete:
        if hasattr(json_payload, "annotated_data"):
            annotated_data = json_payload.annotated_data
        elif isinstance(json_payload, dict) and "annotated_data" in json_payload:
            annotated_data = json_payload["annotated_data"]
        else:
            return json_payload  # No annotated_data found
        
        # Find and remove the annotated data entry with matching ID
        for index, item in enumerate(annotated_data):
            if "id" in item and item["id"] == question_id_to_delete:
                annotated_data.pop(index)
                break
    
    return json_payload


import json
from utils import retry_on_json_error


@retry_on_json_error
def percentage_question(self, prompt):
        schema = """
        {
            "ask_logic": "", # Capture any "ask_if" logic that specifies when a question should be asked.
            "questionLabel": "string",   # Capture the label exactly as it appears.
            "question": "string", # Capture the main question text that we will be showing to the user taking the survey.
            type="percentage"
        }
        """

        parsed_text = self.client_open_ai.chat.completions.create(
            model="gpt-4o",
            temperature=0.0000000001,
            response_format={ "type": "json_object" },
            messages=[
                {"role": "system", "content": "You are a precise survey question parser that captures every detail without losing any information. The most important thing is that we should not delete even a single dot. You are dealing with percentage question type"},
                {"role": "user", "content": f"""
                You are a detailed survey question parser. 

                The schema to be used is: {schema}
                Parsing Rules:
                
                1. Question Label:  
                - Extract the exact identifier at the start (e.g., "Q1.", "5Kn.", etc.).  
                - Keep it exactly as written.  

                2. Question Text:  
                - The question_text in sync_json should be used in the question field of the json.  

                3. Ask If Logic:  
                - Capture any "ask if" condition that determines when a question should be asked.  
                - Store this in the `ask_logic` field.  

                Examples:

                Input:
                ```
                Ask if Q3==0 \n
                QM. Demo percentage Question? Disqualify if <18% \n random text
                ```

                Output:
                {{
                    "ask_logic":"Ask if Q3==0",
                    "questionLabel": "QM",
                    "question": "Demo percentage Question?",
                    "type":"percentage",
                    
                 
                }}
                
 
                
                Important: please give me a valid json.
 
                Parse the following survey question according to these rules: {prompt}
                
                """
                }
            ]
        )

        content = parsed_text.choices[0].message.content
        result = json.loads(content)
        return result


@retry_on_json_error
def percentage_logic(self, input_json,user_logic, input_text):
    

    parsed_text = self.client_open_ai.chat.completions.create(
        model="gpt-4o",
        temperature=0.0000000001,
        response_format={ "type": "json_object" },
        messages=[
            {"role": "system", "content": "You are a precise survey question parser that deals with the logic evaluation."},
            {"role": "user", "content": f"""

            You will receive a JSON containing a percentage question. 
            Your task is to identify and categorize any logic present, ensuring it falls within our supported logic types:

            1. Termination Logics: If the text explicitly indicates termination (e.g., disqualify, end survey etc), assign it accordingly.  
            2. Skip Logics: If the text explicitly mentions skipping to another question, classify it as "Skip" logic.  
            3. User-Defined Logics: Use the predefined logic structure provided by the user.  

            Important:  
            - The user's way of writing logic is: {user_logic}.  
            - Only use explicitly mentioned words as logic do not infer or introduce new logic.  
            - Pay close attention when identifying and assigning logic categories.  

            If there's no logic that we support. (rememebr there can be mutliple ways of writing any logic, we have to be smart enough to understand the bigger picture.).But if we dont support that logic or there is no logic, just return the json as it is. without making any changes.

            But if you see that there's any logic that we support 1. Termination, 2. Skip 3. User defined logics.  then we should follow the below steps.
            In the question if you observe any logic that we support. Then please add another field saying percentage_logic and the key value pair will be the following. 
            For terminating the key should be "Terminating" and the value should be conditon but the condition will only be " [logical operator] [value]" 
            For skip conditions the key would be "SKIP TO [questionLabel of the question]" value would be " [logical operator] [value] " 
            important: The logical operator that you use should be like QuestionLabel ==, >=, <=, != , or whatever the option logic says. every opertor should have questionLabel in front of it. give me the exact logic that you get. you dont have to change any logic. make it both for Terminating or SKIP questions. follow the convention of writing "==, >=, <=, != " even if the user has just one "=" we should eb using "=="
            Important: Provide the exact logic, but include the questionLabel (retrieved from the JSON) before every operator. Ensure the operators are in uppercase, like OR and AND. By questionLabel, I mean the actual value of the questionLabel field from the JSON, not the literal term questionLabel.
           
            If no supported logic is found (remember, there can be multiple ways of writing the same logic, and we need to smartly understand the bigger picture), then do not make any changes to the JSON. If no logic is supported or there is no logic at all, simply return the JSON as it is.
            However, if any logic that we support is identified (1. Termination, 2. Skip, or 3. User-defined logics), please follow these steps:
            1. Adding Logic to the JSON:
            - If you observe any logic that we support, add a new field called `"percentage_logic"` with the following format:
            
                - For Termination Logic:
                - The key should be "Terminating".
                - The value should be the condition, formatted as `"[logical operator] [value]"`.
                
                - For Skip Logic:
                - The key should be `"SKIP TO [questionLabel]"`.
                - The value should be `"[logical operator] [value]"`.
                
            2. Logical Operators:
            - The logical operator should always be written with the questionLabel in front of it, as shown in the example below.
            - Operators such as `==`, `>=`, `<=`, and `!=` must be used, even if the user provided only one `=`.
            - Ensure the operators are in uppercase (e.g., `==`, `>=`, `<=`, `!=`).
            - Do not change the logic; provide the exact logic found in the original input.
            - Do not include any special characters if a number is given for any comparison. (like $, % etc..)

            3. Example of Logic Handling
            - The logic should be formatted as follows:
            
            - For example, if the input JSON is:
                
                {{
                "ask_logic": "Ask if Q3==0",
                "questionLabel": "QM",
                "question": "Demo Question?",
                "type": "percentage"
                }}
                
                
            and the raw text says: (Disqualify if QM=='XYZ')

            - The output JSON should look like this:
                
                {{
                "ask_logic": "Ask if Q3==0",
                "questionLabel": "QM",
                "question": "Demo Question?",
                "type": "percentage",
                "percentage_logic": {{
                    "Terminating": "QM=='XYZ'"
                }}
                }}
               

            4. Additional Notes:
            - Ensure that the `questionLabel` field from the JSON is used as part of the condition (not the literal term `questionLabel`).
            By following this structure, you will ensure consistency and correctness in handling the supported logics.
            example of a logic we dont understand.
            {{
                    "ask_logic":"Ask if Q3==0",
                    "questionLabel": "QM",
                    "question": "Demo Question?",
                    "type":"percentage",
                    
                 
            }}
            Raw text talks about no logic
            we will return it as it is:
            {{
                    "ask_logic":"Ask if Q3==0",
                    "questionLabel": "QM",
                    "question": "Demo Question?",
                    "type":"percentage",
                    
                 
            }}
            Important: There can be multiple logics in the percentage_logic field, so ensure that you add all the logics you can infer from the provided data.
            Please pay special attention to the instructions for the logics and double-check the logic marking to ensure correctness.
            Your task is to return the valid JSON after applying the logics.
      
            The input JSON is: {input_json}
            The input raw text is :{input_text} 
            

           
            """
            }
        ]
    )

    content = parsed_text.choices[0].message.content
    result = json.loads(content)
    return result




import json
from utils import retry_on_json_error

@retry_on_json_error 
def terminate_question(openai_model, prompt):
        schema = """
        {
            "ask_logic": "", # Capture any "ask_if" logic that specifies when a question should be asked.
            "questionLabel": "string",  # Capture the label exactly as it appears.
            "question": "string",  # Capture the main question text that we will be showing to the user taking the survey.
            type="terminate"
        }
        """

        parsed_text = openai_model.client_open_ai.chat.completions.create(
            model="gpt-4o",
            temperature=0.0000000001,
            response_format={ "type": "json_object" },
            messages=[
                {"role": "system", "content": "You are processing survey questions and will receive raw question text. Your task is to structure the question into a JSON format"},
                {"role": "user", "content": f"""
                 
                
                You are a detailed survey question parser. 

                The schema to be used is: {schema}

                Parsing Rules:
                
                1. Question Label:  
                - Extract the exact identifier at the start (e.g., "Q1.", "5Kn.", etc.).  
                - Keep it exactly as written.  

                2. Question Text:  
                - Capture the main question text that we will show to the user taking the survey.  
                - Do not include unnecessary information that is not displayed to the user.  

                3. Ask If Logic:  
                - Capture any "ask if" condition that determines when a question should be asked.  
                - Store this in the `ask_logic` field.  
                
                Examples:

                Input:
                ```
                Ask if Q3==0 \n
                QM. Demo Question? Disqualify if QM=="XYZ" \n random text
                ```

                Output:
                {{
                    "ask_logic":"Ask if Q3==0",
                    "questionLabel": "QM",
                    "question": "Demo Question?",
                    "type":"terminate",
                   
                 
                }}

              
                Important: please give me a valid json.
     
                Parse the following survey question according to these rules: 
                 
                {prompt}
                """
                }
            ]
        )

        content = parsed_text.choices[0].message.content
        result = json.loads(content)
        return result



# def single_text_logic(self, input_json,user_logic):
    

#     parsed_text = self.client_open_ai.chat.completions.create(
#         model="gpt-4o",
#         temperature=0.0000000001,
#         response_format={ "type": "json_object" },
#         messages=[
#             {"role": "system", "content": "You are a precise survey question parser that captures every detail without losing any information."},
#             {"role": "user", "content": f"""
            



#             You will be receving a json which has single_text question , your task is to understand any logics if they exist and put it in the bucket of logics we currently support.
#             we currently just support termination and skip logics on single_text questions.
                
            
#             be sure to smartly understand what the user is trying to mean with that option. There can be multiple ways to say all these logics.
#             Important : Apart from that: Ill be sharing with you the users way of writing a logic for the survey. So you should be smart enough to understand what the user is trying to say and then make our json.
#             Important : The users way of writing the logic is : {user_logic} make sure to only use the words as logics which are explicitly mentioned
#             please pay special attention while assigning logics. 

#             For complete and disqualify -> we will use "THANK AND END" in our final json that we produce and for skip we use "SKIP TO [qstnLabel]" . 

#             In the question if you observe any logic associated with it. please add another field saying text_logic and for the key,value pair. For terminating the key should be Terminating and the value should be conditon but the condition will only be " [logical operator] [value]" and for skip conditions the key would be [ SKIP TO [qstnLabel] value would be " [logical operator] [value] " . 
#             The logical operator that you use should be like QuestionLabel ==, >=, <=, != , or whatever the option logic says. every opertor should have questionLabel in front of it. give me the exact logic that you get. you dont have to change any logic. make it both for Terminating or SKIP questions. Important: give me the exact logic but with questionLabel (pick it up from json) in front of every operator. keep the operators in capital like "OR"."AND". also by questionLabel : I mean the questionLabel of the question. and not "questionLabel".
                
#             Be sure to understand how a user writes the logic. 
#             Use you general understanding and the user_logic way of writing the logic. 
#             Also, if the logic is not in english. please understand it properly
        
#             Very Important: You are not supposed to add any new options on your own, just deal with whatever you have.
             
#             Most Important: No data should be deletd, not even a single dot.
#             Please pay special attention to the instructions for logics and make sure you mark it correctly, double check it. 
#             Important: Donot even delete a single bracket.
#             Most Important: No data should be deletd, not even a single dot.
#             The json is :{input_json}      


                    
#             please give me the correct json. 
#             """
#             }
#         ]
#     )

#     content = parsed_text.choices[0].message.content
#     result = json.loads(content)
#     return result




import json

def terminate_question(openai_model, prompt):
        schema = """
        {
            "ask_logic": "", # Capture any "ask_if" logic that specifies when a question should be asked.
            "questionLabel": "string",  # Capture the label exactly as it appears, even is there's any text till we start the options, should be captured as it is. we dont want to lose any text at all, not even a single dot or bracket.
            "question": "string",  # Full question text with any logic in `{ logic: None }` format . YOU ARE NOT SUPPOSED TO DELETE ANYTHING COPY IT AS IT IS. 
            type="terminate"
        }
        """

        parsed_text = openai_model.client_open_ai.chat.completions.create(
            model="gpt-4o",
            temperature=0.0000000001,
            response_format={ "type": "json_object" },
            messages=[
                {"role": "system", "content": "You are a precise survey question parser that captures every detail without losing any information. The most important thing is that we should not delete even a single dot. You are dealing with transition question type"},
                {"role": "user", "content": f"""
                 
                
                You are a detailed survey question parser. Your task is to analyze survey questions and capture EVERY piece of text while structuring them into a specific JSON format. Follow these guidelines:

                CORE PRINCIPLES:
                1. Never delete or modify any text. WE SHOULD NOT LOSE EVEN A SINGLE DOT. we cannot delete any text at any cost.
                2. Preserve the exact structure and formatting
                3. DONOT EVEN DELETE A SINGLE BRACKET.

                PARSING RULES:

                1. Question Label:
                - Extract the exact identifier at the start (e.g., "Q1.", "5Kn.", etc.)
                - Keep it exactly as written

                2. Question Text:
                - Include everything from the start of the question to just before the options start (if they exist). anything and everything, even if it doent make sense will be a part of the question text.
                - Even random or unrelated text must be part of the question.
                - For any logic/instructions in parentheses or brackets, wrap them in `{{"text": None}}` format
                - Example: "What is your name? " becomes:
                "What is your name? "
                Schema would be: {schema}

                

                Important : Capture any ask if logic in the ask_logic variable. By 'ask if' logic, I mean any statement that determines when a question should be asked. Capture it as it is. 
                Important:  No need to capture ask if ogic in the question text.
                
                SPECIAL INSTRUCTIONS:
                
                
   
                - Keep all original spacing and formatting
                - Never remove any text, even if it seems redundant, we should be capturing each and every detail. we should not delete even a single dot.
                - Capture ALL parenthetical or bracketed instructions in dictionary format.

                the text should be captured as it is, we should not change even a single dot.
                
                CORE PRINCIPLES and MOST IMPORTANT:

                Never delete or modify any text: Every part of the input must be accounted for. Missing even one word is a critical error.
                Preserve structure and format: The output must follow the specified JSON schema exactly.
                please add "type":"terminate" to the json
                The most important thing is that we should not lose any text!!
                Most important: Dont get stuck, if by any chance you're confused give me whatever you understand. just dont delete anything and dont be stuck
                Parse the following survey question according to these rules:  {prompt}
                """
                }
            ]
        )

        content = parsed_text.choices[0].message.content
        result = json.loads(content)
        return result



# def single_text_logic(self, input_json,user_logic):
    

#     parsed_text = self.client_open_ai.chat.completions.create(
#         model="gpt-4o",
#         temperature=0.0000000001,
#         response_format={ "type": "json_object" },
#         messages=[
#             {"role": "system", "content": "You are a precise survey question parser that captures every detail without losing any information."},
#             {"role": "user", "content": f"""
            



#             You will be receving a json which has single_text question , your task is to understand any logics if they exist and put it in the bucket of logics we currently support.
#             we currently just support termination and skip logics on single_text questions.
                
            
#             be sure to smartly understand what the user is trying to mean with that option. There can be multiple ways to say all these logics.
#             Important : Apart from that: Ill be sharing with you the users way of writing a logic for the survey. So you should be smart enough to understand what the user is trying to say and then make our json.
#             Important : The users way of writing the logic is : {user_logic} make sure to only use the words as logics which are explicitly mentioned
#             please pay special attention while assigning logics. 

#             For complete and disqualify -> we will use "THANK AND END" in our final json that we produce and for skip we use "SKIP TO [qstnLabel]" . 

#             In the question if you observe any logic associated with it. please add another field saying text_logic and for the key,value pair. For terminating the key should be Terminating and the value should be conditon but the condition will only be " [logical operator] [value]" and for skip conditions the key would be [ SKIP TO [qstnLabel] value would be " [logical operator] [value] " . 
#             The logical operator that you use should be like QuestionLabel ==, >=, <=, != , or whatever the option logic says. every opertor should have questionLabel in front of it. give me the exact logic that you get. you dont have to change any logic. make it both for Terminating or SKIP questions. Important: give me the exact logic but with questionLabel (pick it up from json) in front of every operator. keep the operators in capital like "OR"."AND". also by questionLabel : I mean the questionLabel of the question. and not "questionLabel".
                
#             Be sure to understand how a user writes the logic. 
#             Use you general understanding and the user_logic way of writing the logic. 
#             Also, if the logic is not in english. please understand it properly
        
#             Very Important: You are not supposed to add any new options on your own, just deal with whatever you have.
             
#             Most Important: No data should be deletd, not even a single dot.
#             Please pay special attention to the instructions for logics and make sure you mark it correctly, double check it. 
#             Important: Donot even delete a single bracket.
#             Most Important: No data should be deletd, not even a single dot.
#             The json is :{input_json}      


                    
#             please give me the correct json. 
#             """
#             }
#         ]
#     )

#     content = parsed_text.choices[0].message.content
#     result = json.loads(content)
#     return result




import requests
from fastapi import Request, HTTPException
from concurrent.futures import ProcessPoolExecutor
import concurrent.futures
from CreateSection.Functions.process_single_question_wrapper import process_single_question_wrapper

async def process_questions_parallel(openai_model_class, all_questions):
        """Process questions using true parallelism with ThreadPoolExecutor"""
        processed_results = {}
        processed_count = 0
        

       

        # Convert questions dict to list of tuples for processing
        question_items = list(all_questions.items())

        # Create a ThreadPoolExecutor instead of ProcessPoolExecutor
        with concurrent.futures.ThreadPoolExecutor() as executor:
            # Submit all tasks with cached AMI check
            futures = []
            for item in question_items:
                future = executor.submit(
                    process_single_question_wrapper,
                    openai_model_class,
                    item,
                )
                futures.append(future)

            # Process results as they complete
            for future in concurrent.futures.as_completed(futures):
                try:
                    key, result = future.result()
                    processed_results[key] = result
                    processed_count += 1

                   
                        
                except Exception as e:
                    print(f"Error processing future: {str(e)}")

      

       
       
        
        

        return processed_results


import json
from utils import retry_on_json_error

@retry_on_json_error
def split_questions(openai_model, prompt):
    parsed_text = openai_model.client_open_ai.chat.completions.create(
        model="gpt-4o",
        temperature=0,  # No need for extreme precision
        response_format={"type": "json_object"},  
        messages=[
            {"role": "system", "content": "You are an extraction agent that must IDENTICALLY copy the input text while separating individual questions."},
            {"role": "user", "content": f"""
             
            Extract every element related to each survey question from the input text and provide a JSON object where the keys are sequential numbers (1, 2, 3, ...) and the values are the full content related to each question. Ensure that no alteration or omission of punctuation occurs, maintaining the complete integrity of the original input text.

            Most important: Not even a single dot should be lost from the input text. I should have each and eveything in the json you output

            # Output Format

            - Provide the extracted content in a JSON format with each segment assigned to a numeric key like 1, 2, 3.

            # Notes

            - Ensure the precision of content extraction, capturing the entire section related to each question.
            - Handle input that doesn't immediately make sense and separate potential questions smartly without losing any details from the original text.
            - Most important: We cannot lose even a single dot from the input text; each and every part should be entirely captured.
            - Capture each and every formatting. we cannot lose anything.
            - everything there in the input text should be there in the json, but splatted into different question.
            - each line break and eveyr formatting should be captured, even if it doesnt make sense. Whatever is there in the input text should be there in the output
             

            The Input text is : {prompt}
            """}
        ]
    )

    try:

        content = parsed_text.choices[0].message.content
        result = json.loads(content)
        return result
    except (KeyError, json.JSONDecodeError) as e:
        print(f"Error parsing response: {e}")
        return None  # Or raise an appropriate exception



import json
from utils import retry_on_json_error

@retry_on_json_error
def categorizer_for_questions_v2(openai_model, prompt):
        

        parsed_text = openai_model.client_open_ai.chat.completions.create(
            model="gpt-4o",
            temperature=0.0000000001,
            response_format={ "type": "json_object" },
            messages=[
                {"role": "system", "content": "you are a specialised agent which can categorize a given question into a type."},
                {"role": "user", "content": f"""
                
                The categories that we support are:
                single-select
                multi-select
                single-text
                multi-text
                grid
                number
                currency
                year
                percentage
                percent-sum
                transition
                notes
                nps
                hidden_variable
                maxdiff
                contact-form
                ranking
                multi-percentage
                van-westendorp
                zipcode
                ai-chat
            
                
                Guidelines:
                
                Determine the question type based on the input text.
                Net Promoter Score (NPS): If the question is a "Net Promoter Score" question, mark its type as nps.
                number: Questions like "How old are you?" should be classified as number.
                
                Message vs Notes:
                transition: Text intended to be shown to the user, such as introductions or filler questions.
                Notes: Programmer notes intended for creating the survey.
                    
                Any hidden variable question will be type variable.
                 
                Any question which has options should be routed accordingly, make sure to be smart enough to decide. 
                 
               
                If the question type is not part of the supported categories or falls under any of the following categories, please mark it as 'unknown': Conjoint, Carousel, Embedded Data, Semantic Differential, Captcha, Media Upload (image, voice, video), Heatmap, Highlighter, Implicit Association, Slider scales for response.
                
                    
                Output Format:
                Provide the response as a JSON object with a single key-value pair:

                Key: "type"
                Value: The identified type.
                Example Output:

                    {{
                    "type": "single_select"
                    }}
                    
                    The question text is : {prompt}

                """
                }
            ]
        )

        content = parsed_text.choices[0].message.content
        result = json.loads(content)
        return result





from CreateSection.Functions.processor_map_sync import *
import json
from CreateSection.Question_TypesV2.format import format_text
import requests
from CreateSection.Question_TypesV2.categorizer_v2 import categorizer_for_questions_v2
from CreateSection.Functions.annotation import annotation




def process_single_question_wrapper(openai_model_class, question_data):
    """Process a single question in a separate process"""
    
    
    key, original_value = question_data
    cleaned_original_value = original_value
    annotation_result = {"nodes": []} 
    
    try:
        openai_model = openai_model_class()
        
        if not key.endswith('_d'):
            if key.endswith('_Section'):
                return key, str(original_value)
            
            return key, original_value

 
        processor_map = get_processor_map_sync()
        question_type_identified= categorizer_for_questions_v2(openai_model,original_value)
        
        print("the question type identified is : ",question_type_identified)
        question_type = question_type_identified.get("type", "unknown")
        
        if question_type in processor_map:
            
            
            
            # if use_ami:
            #     annotation_result= annotation_ami(openai_model,cleaned_original_value,keymap_names,question_type)   
            # else:
            annotation_result= annotation(openai_model,original_value,question_type)    
      
            annotation_result["nodes"].extend([
                {'type': 'linebreak'},
                {'type': 'linebreak'},
                {'type': 'linebreak'}
            ])
      
            question_func, logic_func = processor_map[question_type]

            
            one = question_func(openai_model, original_value)
            
            if logic_func:
                final_json= logic_func(openai_model, one, original_value)
            else:
                final_json=one
            
            if final_json.get("ask_logic"):
                print("the input for ask if logic is :",final_json)
                final_json = ask_if_logic(openai_model, final_json)
            
        else:
            final_json = {
                "question": original_value,
                "questionLabel": "",
                "type": "unknown"
            }

        final_json['raw_data'] = cleaned_original_value
        final_json['annotated_text'] = annotation_result
        return key, final_json

    except Exception as e:
        print(f"Error processing question haha {key}: {str(e)}")
        return key, {
            "questionLabel": f"Unknown_{key}",
            "question": str(original_value) if isinstance(original_value, str) else json.dumps(original_value),
            "type": "unknown",
            "options": {},
            "columns": {},
            "rows": {},
            "logic": [],
            "Randomize": False,
            "raw_data": str(original_value)
        }



import json

def categorizer_for_questions(openai_model, prompt):
        

        parsed_text = openai_model.client_open_ai.chat.completions.create(
            model="gpt-4o",
            temperature=0.0000000001,
            response_format={ "type": "json_object" },
            messages=[
                {"role": "system", "content": "you are a specialised agent which can categorize a given question into a type."},
                {"role": "user", "content": f"""
                
              The categories that we support are:
              single_select
              multi_select
              single_text
              multi_text
              grid
              number
              currency
              year
              percentage
              percent_sum
              message
              notes
              nps
              hidden_variable
              maxdiff
              contact_form
              ranking
              multi_percentage
              van_westerndorp
              zip_code
              ai_chat
            
                
              
              Guidelines:
              
              Determine the question type based on the provided options.
              Net Promoter Score (NPS): If the question is a "Net Promoter Score" question, mark its type as nps.
              numbers: Questions like "How old are you?" should be classified as number.
              
              Message vs Notes:
              message: Text intended to be shown to the user, such as introductions or filler questions.
              Notes: Programmer notes intended for creating the survey.
                 

             
              If the question type is not in the supported categories, tag it as unknown.
              
                 
              Output Format:
              Provide the response as a JSON object with a single key-value pair:

              Key: "type"
              Value: The identified type.
              Example Output:

                {{
                  "type": "single_select"
                }}
                
                The question text is : {prompt}

                """
                }
            ]
        )

        content = parsed_text.choices[0].message.content
        result = json.loads(content)
        return result


